#############################################################################
### PROJECT: Differential methylation analysis of Infant leukemias
### and differentiated blood cells with 850k arrays
###
### Script for the analysis of WGBS methylation changes in ALL subtypes
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

# Set up the directory for the results:
tablesDirectory <- ""
resultsDirectory <- ""

library(bsseq)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(annotatr)
library(RColorBrewer)
library(scales)

# Load the region files (create file with the regions of interest):
custom_dmrs <- read.csv(paste(tablesDirectory, "custom_coordinates_FTL3.csv", sep = "/"), sep = ",", stringsAsFactors = F)

# Vector to keep standard chromosomes:
chrnames <- c("chr1", "chr2", "chr3", "chr4", "chr5", "chr6", "chr7", "chr8", "chr9", "chr10", "chr11", "chr12", "chr13", "chr14", "chr15", "chr16", "chr17", "chr18", "chr19", "chr20", "chr21", "chr22", "chrX", "chrY")

# Get annotation: 
getAnnot <- function(genomeName) {
  requireNamespace("annotatr")
  liftTo <- NULL
  if(genomeName == 'hg18'){
    message("Genome ", genomeName, " will be built by lifting over ",
            "hg19 annotations from annotatr")
    liftTo <- 'hg18'
    genomeName <- 'hg19'
  }else if (!genomeName %in% annotatr::builtin_genomes()) {
    message("Genome ", genomeName, " is not supported by ", 
            "annotatr at this time")
    return(NULL)
  }
  
  if (is.null(genomeName)) {
    return(NULL)
  } else {
    annot_CpG <- paste0(c(genomeName, "_cpgs"), collapse = "")
    annot_genes <- paste0(c(genomeName, "_genes_cds"), collapse = "")
    
    # Build the annotations (a single GRanges object)
    
    # annotatr downloads files for genomes that aren't natively supported
    # (e.g. hg38)
    # Download has a nonzero fail rate; allow up to 5 retries before 
    # throwing an error
    
    for (attempt in seq_len(5)) {
      cpg <- try(annotatr::build_annotations(genome = genomeName, 
                                             annotations = annot_CpG), 
                 silent = TRUE)
      if (!is(cpg, "try-error")) {
        message("Download of CpG annotation successful!")
        fail1 <- 0
        break
      } else {
        message(cpg)
        if (5 - attempt > 0)
          message("Trying again (", 5 - attempt, " attempts remaining)")
        fail1 <- 1
      }
    }
    
    for (attempt in seq_len(5)) {
      genes <- try(annotatr::build_annotations(genome = genomeName,
                                               annotations = annot_genes), 
                   silent = TRUE)
      if (!is(genes, "try-error")) {
        message("Download of Gene annotation successful!")
        fail2 <- 0
        break
      } else {
        message(genes)
        if (5 - attempt > 0)
          message("Trying again (", 5 - attempt, " attempts remaining)")
        fail2 <- 1
      }
    }
    
    
    if (fail1 == 0 && fail2 == 0) {
      # lift over hg19 coordinates to hg18 using annotationHub
      if (!is.null(liftTo)){
        ah = AnnotationHub()
        chainfiles <- query(ah , c(genomeName, liftTo, "chainfile"))
        cf <- which(grepl(paste0(genomeName, "To", liftTo), 
                          chainfiles$title, 
                          ignore.case=TRUE))
        if (length(cf) == 0){
          stop("LiftOver from ", genomeName, " to ", liftTo, 
               " was unsucccessful")
        }else if(length(cf) > 1){
          # take the first matching chain if more than one
          cf <- cf[1]
        }
        
        chain <- chainfiles[[names(chainfiles)[cf]]]
        
        cpg.new <- unlist(liftOver(cpg, chain))
        genes.new <- unlist(liftOver(genes, chain))
        
        GenomeInfoDb::genome(cpg) <- liftTo
        GenomeInfoDb::genome(genes) <- liftTo
      }
      
      keep <- which(!is.na(genes$symbol))
      genes <- genes[keep, ]
      cpg$type <- substr(cpg$type, 10, nchar(cpg$type))
      
      annot <- GenomicRangesList(CpGs = cpg, Exons = genes)
      return(annot)
    } else {
      stop("Annotation could not be retrieved.")
    }
  }
}

###############################################

plotAnnoTrack <- function(gr, annoTrack) {
  # Code adapted from bsseq package
  
  ## check may need to be modified
  if (!is(annoTrack, "SimpleGenomicRangesList"))
    stop("'annoTrack' needs to be a 'SimpleGenomicRangesList'")
  plot(start(gr), 1, type = "n", xaxt = "n", yaxt = "n", bty = "n", 
       ylim = c(0, length(annoTrack) + 0.5), xlim = c(start(gr), end(gr)), 
       xlab = "", ylab = "")
  
  # add legend
  vars <- list(Island = "Island   ", Shore = "Shore   ", Shelf = "Shelf   ", 
               OpenSea1 = "Open ", OpenSea2 = "Sea")
  cols <- c("forestgreen", "goldenrod2", "dodgerblue", "blue3", "blue3")
  for (i in seq_len(length(vars))) {
    tmpvars <- vars
    tmpvars[-i] <- paste("phantom('", tmpvars[-i], "')", sep = "")
    expr <- paste(tmpvars, collapse = "*")
    text(start(gr), 1.8, parse(text = expr), col = cols[i], adj = c(0, 1),
         cex = 0.85)
  }
  
  lapply(seq(along = annoTrack), function(ii) {
    jj <- length(annoTrack) + 1 - ii
    ir <- subsetByOverlaps(annoTrack[[ii]], gr)
    start(ir) <- pmax(start(ir), start(gr))
    end(ir) <- pmin(end(ir), end(gr))
    
    if (ii == 2) {
      jj <- jj - 0.15
    }
    
    if (length(ir) > 0) {
      if (ii == 2) {
        colourCount <- length(unique(ir$symbol))
        getPalette <- colorRampPalette(alpha(brewer.pal(max(
          length(unique(ir$symbol)), 
          3), "Dark2"), 0.4))
        color.pal <- getPalette(colourCount)
        names(color.pal) <- unique(ir$symbol)
        map <- match(ir$symbol, names(color.pal))
        color <- color.pal[map]
        
        bord <- "black"
      } else if (ii == 1) {
        color <- ir$type
        color[color == "inter"] <- "blue3"
        color[color == "shelves"] <- "dodgerblue"
        color[color == "shores"] <- "goldenrod2"
        color[color == "islands"] <- "forestgreen"
        bord <- color
        
        rect(start(ir), jj - 0.06, end(ir), jj + 0.17, col = color, 
             border = bord)
      }
      
      if (ii == 2) {
        lastPos <- rep(NA, length(unique(ir$symbol)) - 1)
        used <- NULL
        for (k in seq_len(length(unique(ir$symbol)))) {
          irk <- ir[ir$symbol == unique(ir$symbol)[k], ]
          rect(min(start(irk)), jj - 0.065, max(end(irk)), jj + 0.065, 
               col = alpha("black", 
                           0.1), border = alpha("black", 0.1))
          if (!(unique(ir$symbol)[k] %in% used)) {
            rwidth <- end(gr) - start(gr)
            sg <- pmin(start(irk), end(irk))
            eg <- pmax(start(irk), end(irk))
            gwidth <- min(max(eg), end(gr)) - max(min(sg), start(gr))
            textPos <- max(min(sg), start(gr)) + gwidth/2
            jj.orig <- jj
            if (sum(!is.na(lastPos)) > 0) {
              separation <- (textPos - lastPos[k - 1])/rwidth
              if (abs(separation) <= 0.2 && k < 3) {
                jj <- jj - 0.29
              } else {
                separation <- min(abs((textPos - lastPos)/rwidth), 
                                  na.rm = TRUE)
                if (abs(separation) <= 0.2) {
                  jj <- jj - 0.29
                }
              }
            }
            lastPos[k] <- textPos
            text(textPos, jj - 0.375, labels = unique(irk$symbol), 
                 cex = 0.85, 
                 col = unique(color)[k])
            jj <- jj.orig
            used <- c(used, unique(ir$symbol)[k])
          }
          rect(sg, jj - 0.11, eg, jj + 0.12, col = unique(color)[k], 
               border = bord)
        }
      }
      
    }
    mtext(names(annoTrack)[ii], side = 2, at = jj, las = 1, line = 1)
  })
  
}

###############################################

plotGeneTrack <- function(gr, geneTrack, cex) {
  geneTrack_gr <- makeGRangesFromDataFrame(geneTrack)
  ol <- findOverlaps(geneTrack_gr, gr)
  genes <- geneTrack[queryHits(ol), ]
  plot(start(gr), 1, type = "n", xaxt = "n", yaxt = "n", bty = "n",
       ylim = c(-1.5, 1.5), xlim = c(start(gr), end(gr)),
       xlab = "", ylab = "", cex.lab = 4, lheight = 2, cex.axis = 1)
  if (nrow(genes) > 0) {
    for (g in 1:nrow(genes)) {
      geneind2 = which(geneTrack$symbol == genes$symbol[g])
      geneind2 = geneind2[which(geneTrack$tx_id[geneind2] == 1)]
      direction = unique(geneTrack$strand[geneind2])
      ES = geneTrack$start[geneind2]
      EE = geneTrack$end[geneind2]
      Exons = cbind(ES, EE)
      if (direction == "+") {
        lines(x = c(min(ES), max(EE)),
              y = c(0.65, 0.65))
        apply(Exons, 1, function(x) {
          polygon(c(x[1], x[2], x[2], x[1]),
                  c(0.45, 0.45, 0.85, 0.85), col = "darkgrey")
        })
        text((max(start(gr), min(ES)) +
                min(end(gr), max(EE))) / 2, 1.2,
             genes$symbol[g], cex = cex)
      } else {
        lines(x = c(min(ES), max(EE)),
              y = c(-0.65, -0.65))
        apply(Exons, 1, function(x)
          polygon(c(x[1], x[2], x[2], x[1]),
                  c(-0.45, -0.45, -0.85, -0.85),
                  col = "darkgrey"))
        text((max(start(gr), min(ES)
        ) + min(end(gr), max(EE)
        )) / 2, -1.2, genes$symbol[g], cex = cex)
      }
      
    }
  }
}

###############################################

.plotSmoothData <- function(BSseq, region, extend, addRegions, col, lty, lwd, regionCol,
                            addTicks, addPoints, pointsMinCov, highlightMain) {
  gr <- .bsGetGr(BSseq, region, extend)
  BSseq <- subsetByOverlaps(BSseq, gr)
  
  ## Extract basic information
  sampleNames <- sampleNames(BSseq)
  names(sampleNames) <- sampleNames
  positions <- start(BSseq)
  smoothPs <- getMeth(BSseq, type = "smooth")
  rawPs <- getMeth(BSseq, type = "raw")
  coverage <- getCoverage(BSseq)
  
  # Realise in memory data that are to be plotted
  if (addPoints) {
    rawPs <- as.array(rawPs)
    coverage <- as.array(coverage)
  }
  smoothPs <- as.array(smoothPs)
  
  ## get col, lwd, lty
  colEtc <- .bsGetCol(object = BSseq, col = col, lty = lty, lwd = lwd)
  
  ## The actual plotting
  plot(positions[1], 0.5, type = "n", xaxt = "n", yaxt = "n",
       ylim = c(0,1), xlim = c(start(gr), end(gr)), xlab = "", ylab = "Methylation")
  axis(side = 2, at = c(0.2, 0.5, 0.8))
  if(addTicks)
    rug(positions)
  
  .bsHighlightRegions(regions = addRegions, gr = gr, ylim = c(0,1),
                      regionCol = regionCol, highlightMain = highlightMain)
  
  if(addPoints) {
    sapply(1:ncol(BSseq), function(sampIdx) {
      abline(v = positions[rawPs[, sampIdx] > 0.1], col = "grey80", lty = 1)
    })
  } # This adds vertical grey lines so we can see where points are plotted
  
  sapply(1:ncol(BSseq), function(sampIdx) {
    .bsPlotLines(positions, smoothPs[, sampIdx], col = colEtc$col[sampIdx],
                 lty = colEtc$lty[sampIdx], lwd = colEtc$lwd[sampIdx],
                 plotRange = c(start(gr), end(gr)))
  })
  
  if(addPoints) {
    sapply(1:ncol(BSseq), function(sampIdx) {
      .bsPlotPoints(positions, rawPs[, sampIdx], coverage[, sampIdx],
                    col = colEtc$col[sampIdx], pointsMinCov = pointsMinCov)
    })
  }
}

###############################################
.bsPlotLines <- function(x, y, col, lty, lwd, plotRange) {
  if(sum(!is.na(y)) <= 1)
    return(NULL)
  xx <- seq(from = plotRange[1], to = plotRange[2], length.out = 500)
  yy <- approxfun(x, y)(xx)
  lines(xx, yy, col = col, lty = lty, lwd = lwd)
}

.bsPlotPoints <- function(x, y, z, col, pointsMinCov) {
  points(x[z>pointsMinCov], y[z>pointsMinCov], col = col, pch = 16, cex = 0.5)
}

.bsHighlightRegions <- function(regions, gr, ylim, regionCol, highlightMain) {
  if(is.data.frame(regions))
    regions <- data.frame2GRanges(regions)
  if(highlightMain)
    regions <- c(regions, gr)
  if(is.null(regions)) return(NULL)
  ## regions <- pintersect(region, rep(gr, length(regions)))
  ## regions <- regions[width(regions) == 0]
  regions <- subsetByOverlaps(regions, gr)
  regions <- pintersect(regions, rep(gr, length(regions)))
  if(length(regions) == 0)
    return(NULL)
  rect(xleft = start(regions), xright = end(regions), ybottom = ylim[1],
       ytop = ylim[2], col = regionCol, border = NA)
}

.bsGetCol <- function(object, col, lty, lwd) {
  ## Assumes that object has pData and sampleNames methods
  if(is.null(col)) {
    if("col" %in% names(pData(object)))
      col <- pData(object)[["col"]]
    else
      col <- rep("black", nrow(pData(object)))
  }
  if(length(col) != ncol(object))
    col <- rep(col, length.out = ncol(object))
  if(is.null(names(col)))
    names(col) <- sampleNames(object)
  
  if(is.null(lty)) {
    if("lty" %in% names(pData(object)))
      lty <- pData(object)[["lty"]]
    else
      lty <- rep(1, ncol(object))
  }
  if(length(lty) != ncol(object))
    lty <- rep(lty, length.out = ncol(object))
  if(is.null(names(lty)))
    names(lty) <- sampleNames(object)
  
  if(is.null(lwd)) {
    if("lwd" %in% names(pData(object)))
      lwd <- pData(object)[["lwd"]]
    else
      lwd <- rep(1, nrow(pData(object)))
  }
  if(length(lwd) != ncol(object))
    lwd <- rep(lwd, length.out = ncol(object))
  if(is.null(names(lwd)))
    names(lwd) <- sampleNames(object)
  
  return(list(col = col, lty = lty, lwd = lwd))
}

.bsPlotTitle <- function(gr, extend, main, mainWithWidth) {
  if(is.data.frame(gr))
    gr <- data.frame2GRanges(gr)
  if(length(gr) > 1) {
    warning("plotTitle: gr has more than one element")
    gr <- gr[1]
  }
  plotChr <- as.character(seqnames(gr))
  plotRange <- c(start(gr), end(gr))
  regionCoord <- sprintf("%s: %s - %s", plotChr,
                         format(plotRange[1], big.mark = ",", scientific = FALSE),
                         format(plotRange[2], big.mark = ",", scientific = FALSE))
  if(mainWithWidth) {
    regionWidth <- sprintf("width = %s, extended = %s",
                           format(width(gr), big.mark = ",", scientific = FALSE),
                           format(extend, big.mark = ",", scientific = FALSE))
    regionCoord <- sprintf("%s (%s)", regionCoord, regionWidth)
  }
  if(main != "") {
    main <- sprintf("%s\n%s", main, regionCoord)
  } else {
    main <- regionCoord
  }
  main
}

.bsGetGr <- function(object, region, extend) {
  if(is.null(region)) {
    gr <- GRanges(seqnames = seqnames(object)[1],
                  ranges = IRanges(start = min(start(object)),
                                   end = max(start(object))))
  } else {
    if(is(region, "data.frame"))
      gr <- data.frame2GRanges(region, keepColumns = FALSE)
    else
      gr <- region
    if(!is(gr, "GRanges") || length(gr) != 1)
      stop("'region' needs to be either a 'data.frame' (with a single row) or a 'GRanges' (with a single element)")
    gr <- resize(gr, width = 2*extend + width(gr), fix = "center")
  }
  gr
}

###############################################
plotRegion <- function(BSseq, region = NULL, extend = 0, main = "", addRegions = NULL,
                       annoTrack = NULL,
                       geneTrack = NULL, cex.gene = 1.5,
                       col = NULL, lty = NULL, lwd = NULL,
                       BSseqStat = NULL, stat = "tstat.corrected", stat.col = "black",
                       stat.lwd = 1, stat.lty = 1, stat.ylim = c(-8,8),
                       mainWithWidth = TRUE,
                       regionCol = alpha("red", 0.1), addTicks = TRUE, addPoints = FALSE,
                       pointsMinCov = 5, highlightMain = FALSE) {
  
  opar <- par(mar = c(0,4.1,0,0), oma = c(5,0,4,2), mfrow = c(1,1))
  on.exit(par(opar))
  if(is.null(BSseqStat) && is.null(geneTrack)) {
    layout(matrix(1:2, ncol = 1), heights = c(2,1))
  } else if (is.null(geneTrack)) {
    layout(matrix(1:3, ncol = 1), heights = c(2,2,1))
  } else {
    layout(matrix(1:4, ncol = 1), heights = c(2,2,1,0.3))
  }
  
  .plotSmoothData(BSseq = BSseq, region = region, extend = extend, addRegions = addRegions,
                  col = col, lty = lty, lwd = lwd, regionCol = regionCol,
                  addTicks = addTicks, addPoints = addPoints,
                  pointsMinCov = pointsMinCov, highlightMain = highlightMain)
  gr <- .bsGetGr(BSseq, region, extend)
  
  if(!is.null(BSseqStat)) {
    BSseqStat <- subsetByOverlaps(BSseqStat, gr)
    if(is(BSseqStat, "BSseqTstat")) {
      stat.values <- as.array(getStats(BSseqStat)[, "tstat.corrected"])
      stat.values <- as.array(stat.values)
      stat.type <- "tstat"
    }
    if(is(BSseqStat, "BSseqStat")) {
      stat.type <- getStats(BSseqStat, what = "stat.type")
      if(stat.type == "tstat") {
        stat.values <- getStats(BSseqStat, what = "stat")
        stat.values <- as.array(stat.values)
      }
      if(stat.type == "fstat") {
        stat.values <- sqrt(getStats(BSseqStat, what = "stat"))
        stat.values <- as.array(stat.values)
      }
    }
    plot(start(gr), 0.5, type = "n", xaxt = "n", yaxt = "n",
         ylim = stat.ylim, xlim = c(start(gr), end(gr)), xlab = "", ylab = stat.type)
    axis(side = 2, at = c(-5,0,5))
    abline(h = 0, col = "grey60")
    .bsPlotLines(start(BSseqStat), stat.values, lty = stat.lty, col = stat.col, lwd = stat.lwd,
                 plotRange = c(start(gr), end(gr)))
  }
  
  if(!is.null(annoTrack))
    plotAnnoTrack(gr, annoTrack)
  
  if (!is.null(geneTrack))
    plotGeneTrack(gr, geneTrack)
  
  if(!is.null(main)) {
    main <- .bsPlotTitle(gr = region, extend = extend, main = main,
                         mainWithWidth = mainWithWidth)
    mtext(side = 3, text = main, outer = TRUE, cex = 1)
  }
  return(invisible(NULL))
}

###############################################

read.lister <- function(file){
  # Read and tune up the file:
  dat <- read.csv(file = file, sep = "\t", header = T)
  colnames(dat) = c("chrBase", "chr",	"base",	"strand",	"coverage",	"freqC",	"freqT")
  # Convert columns to corresponding classes:
  nms <- c(1,2,4)
  dat[nms] <- lapply(dat[nms], as.character) 
  nms <- c(3,5)
  dat[nms] <- lapply(dat[nms], as.integer)
  nms <- c(6,7)
  dat[nms] <- lapply(dat[nms], as.numeric)
  # Tune up the strand:
  dat$strand[which(dat$strand == "F")] <- "+"
  dat$strand[which(dat$strand == "R")] <- "-"
  # Keep standard chromosomes:
  dat$chr <- paste("chr", dat$chr, sep = "")
  dat <- dat[which(dat$chr %in% chrnames),]
  dat$M <- as.integer(round((dat$coverage*dat$freqC)/100, 0))
  # Keep info from both strands:
  tmp <- dat[dat$strand == "+",]
  BS.forward <- BSseq(pos = tmp$base, chr = tmp$chr, M = as.matrix(tmp$M, ncol = 1),
                      Cov = as.matrix(tmp$coverage, ncol = 1), sampleNames = "forward")
  tmp <- dat[dat$strand == "-",]
  BS.reverse <- BSseq(pos = tmp$base - 1L, chr = tmp$chr, M = as.matrix(tmp$M, ncol = 1),
                      Cov = as.matrix(tmp$coverage, ncol = 1), sampleNames = "reverse")
  BS <- combine(BS.forward, BS.reverse)
  BS <- collapseBSseq(BS, group = c("a", "a"))
  BS
}

BS.ProB.r1 <- read.lister(paste(tablesDirectory, "WGBS_analyses/BCP_1_CpG.txt", sep = "/"))
sampleNames(BS.ProB.r1) <- "BCP_1"
BS.ProB.r2 <- read.lister(paste(tablesDirectory, "WGBS_analyses/BCP_2_CpG.txt", sep = "/"))
sampleNames(BS.ProB.r2) <- "BCP_2"
BS.AF4.r1 <- read.lister(paste(tablesDirectory, "WGBS_analyses/MLL-AF4_1_CpG.txt", sep = "/"))
sampleNames(BS.AF4.r1) <- "MLL-AF4_1"
BS.AF4.r2 <- read.lister(paste(tablesDirectory, "WGBS_analyses/MLL-AF4_2_CpG.txt", sep = "/"))
sampleNames(BS.AF4.r2) <- "MLL-AF4_2"
BS.AF9.r1 <- read.lister(paste(tablesDirectory, "WGBS_analyses/MLL-AF9_1_CpG.txt", sep = "/"))
sampleNames(BS.AF9.r1) <- "MLL-AF9_1"
BS.AF9.r2 <- read.lister(paste(tablesDirectory, "WGBS_analyses/MLL-AF9_2_CpG.txt", sep = "/"))
sampleNames(BS.AF9.r2) <- "MLL-AF9_2"
BS.nonMLL.r1 <- read.lister(paste(tablesDirectory, "WGBS_analyses/non-MLL_1_CpG.txt", sep = "/"))
sampleNames(BS.nonMLL.r1) <- "nonMLL_1"
BS.nonMLL.r2 <- read.lister(paste(tablesDirectory, "WGBS_analyses/non-MLL_2_CpG.txt", sep = "/"))
sampleNames(BS.nonMLL.r2) <- "nonMLL_2"

BS.cancer <- combine(BS.ProB.r1, BS.ProB.r2, BS.AF4.r1, BS.AF4.r2, BS.AF9.r1, BS.AF9.r2, BS.nonMLL.r1, BS.nonMLL.r2)
pData(BS.cancer)$Rep <- c("replicate1", "replicate2", "replicate1", "replicate2", "replicate1", "replicate2", "replicate1", "replicate2")
pData(BS.cancer)$col <- c("#adb7d6", "#adb7d6", "#D53E4F", "#D53E4F", "#2eaa6c", "#2eaa6c", "#e8db4c", "#e8db4c")
validObject(BS.cancer)
pData(BS.cancer)
BS.cancer.smooth <- BSmooth(BS.cancer, BPPARAM = MulticoreParam(workers = 1), verbose = TRUE)


# Plot DMRs:
# Load the annotation files
annoTrack <- getAnnot("hg19")
pData(BS.cancer.smooth) <- pData(BS.cancer)

pdf(file = paste(resultsDirectory, "DMRs_bsseq_FTL3.pdf", sep = "/"), paper = "a4r", height = 21, width = 28, onefile = T)
plotRegion(BS.cancer.smooth, custom_dmrs[1,], extend = 1000, annoTrack = annoTrack, main = "FTL3")
dev.off()
