#############################################################################
### PROJECT: Differential methylation analysis of Infant leukemias
### and differentiated blood cells with 850k arrays
###
### Script for the analysis of WGBS methylation changes in ALL subtypes
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

# Set up the directory for the results:
WGBSDirectory <- ""
tablesDirectory <- ""
resultsDirectory <- ""

library(bsseq)

setwd(WGBSDirectory)


# Vector to keep standard chromosomes:
chrnames <- c("chr1", "chr2", "chr3", "chr4", "chr5", "chr6", "chr7", "chr8", "chr9", "chr10", "chr11", "chr12", "chr13", "chr14", "chr15", "chr16", "chr17", "chr18", "chr19", "chr20", "chr21", "chr22", "chrX", "chrY")

read.lister <- function(file){
  # Read and tune up the file:
  dat <- read.csv(file = file, sep = "\t", header = T)
  colnames(dat) = c("chrBase", "chr",	"base",	"strand",	"coverage",	"freqC",	"freqT")
  # Convert columns to corresponding classes:
  nms <- c(1,2,4)
  dat[nms] <- lapply(dat[nms], as.character) 
  nms <- c(3,5)
  dat[nms] <- lapply(dat[nms], as.integer)
  nms <- c(6,7)
  dat[nms] <- lapply(dat[nms], as.numeric)
  # Tune up the strand:
  dat$strand[which(dat$strand == "F")] <- "+"
  dat$strand[which(dat$strand == "R")] <- "-"
  # Keep standard chromosomes:
  dat$chr <- paste("chr", dat$chr, sep = "")
  dat <- dat[which(dat$chr %in% chrnames),]
  dat$M <- as.integer(round((dat$coverage*dat$freqC)/100, 0))
  # Keep info from both strands:
  tmp <- dat[dat$strand == "+",]
  BS.forward <- BSseq(pos = tmp$base, chr = tmp$chr, M = as.matrix(tmp$M, ncol = 1),
                      Cov = as.matrix(tmp$coverage, ncol = 1), sampleNames = "forward")
  tmp <- dat[dat$strand == "-",]
  BS.reverse <- BSseq(pos = tmp$base - 1L, chr = tmp$chr, M = as.matrix(tmp$M, ncol = 1),
                      Cov = as.matrix(tmp$coverage, ncol = 1), sampleNames = "reverse")
  BS <- combine(BS.forward, BS.reverse)
  BS <- collapseBSseq(BS, group = c("a", "a"))
  BS
}
BS.nonMLL.r1 <- read.lister(paste(WGBSDirectory, "non-MLL_1_CpG.txt", sep = "/"))
sampleNames(BS.nonMLL.r1) <- "non-MLL_1"
BS.nonMLL.r2 <- read.lister(paste(WGBSDirectory, "non-MLL_2_CpG.txt", sep = "/"))
sampleNames(BS.nonMLL.r2) <- "non-MLL_2"
BS.BCP.r1 <- read.lister(paste(WGBSDirectory, "BCP_1_CpG.txt", sep = "/"))
sampleNames(BS.BCP.r1) <- "BCP_1"
BS.BCP.r2 <- read.lister(paste(WGBSDirectory, "BCP_2_CpG.txt", sep = "/"))
sampleNames(BS.BCP.r2) <- "BCP_2"

BS.cancer <- combine(BS.nonMLL.r1, BS.nonMLL.r2, BS.BCP.r1, BS.BCP.r2)
pData(BS.cancer)$Rep <- c("replicate1", "replicate2", "replicate1", "replicate2")
validObject(BS.cancer)
pData(BS.cancer)

# Get smooth data:
BS.cancer.smooth <- BSmooth(BS.cancer, verbose = TRUE, mc.cores = 12)

# Compute T-statistics:
BS.cancer.ex.tstat <- BSmooth.tstat(BS.cancer.smooth, group1 = c("non-MLL_1", "non-MLL_2"), group2 = c("BCP_1", "BCP_2"), estimate.var = "paired", local.correct = TRUE, verbose = TRUE)
plot(BS.cancer.ex.tstat)

# Get quantiles:
tstats <- as.data.frame(BS.cancer.ex.tstat@stats[,6])
colnames(tstats) <- "tstats"
quantiles <- quantile(tstats$tstats)

# Find DMRs:
dmrs0 <- dmrFinder(BS.cancer.ex.tstat, cutoff = c(quantiles[[2]], quantiles[[4]]))
dmrs <- subset(dmrs0, n >= 3 & abs(meanDiff) >= 0.2)

# Plot DMRs:
pData <- pData(BS.cancer.smooth)
pData$col <- rep(c("red", "blue"), each = 2)
pData(BS.cancer.smooth) <- pData

plotRegion(BS.cancer.smooth, dmrs[1,], extend = 5000, addRegions = dmrs)

pdf(file = paste(resultsDirectory, "dmrs_top200_non_MLL.pdf", sep = "/"), width = 10, height = 5)
plotManyRegions(BS.cancer.smooth, dmrs[1:50,], extend = 5000, addRegions = dmrs)
dev.off()

# Write DMRs to file:
write.table(dmrs, file = paste(tablesDirectory,"dmrs_Bsseq_B20_non_MLL.txt", sep = "/"), sep = "\t", quote = F, col.names = T, row.names = F)
