#############################################################################
### PROJECT: Differential methylation analysis of Infant leukemias
### and differentiated blood cells with 850k arrays
###
### Script for the analysis of WGBS methylation changes in ALL subtypes
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

# Set up the directory for the results:
tablesDirectory <- ""
resultsDirectory <- ""

library(dplyr)
library(tidyr)
library(ggplot2)
library(reshape2)
library(pheatmap)
library(RColorBrewer)
library(scales)
library(gridExtra)
library(grid)

tablesDirectory <- ""
resultsDirectory <- ""
setwd(tablesDirectory)

hyper_AF4 <- read.csv(file = paste(tablesDirectory,"enrichment_repeats_hyper_DMRs_AF4.txt", sep = ""), sep = "\t")
hyper_AF4$userSet <- "AF4_hyper"
hypo_AF4 <- read.csv(file = paste(tablesDirectory,"enrichment_repeats_hypo_DMRs_AF4.txt", sep = ""), sep = "\t")
hypo_AF4$userSet <- "AF4_hypo"
hyper_AF9 <- read.csv(file = paste(tablesDirectory,"enrichment_repeats_hyper_DMRs_AF9.txt", sep = ""), sep = "\t")
hyper_AF9$userSet <- "AF9_hyper"
hypo_AF9 <- read.csv(file = paste(tablesDirectory,"enrichment_repeats_hypo_DMRs_AF9.txt", sep = ""), sep = "\t")
hypo_AF9$userSet <- "AF9_hypo"
hyper_nonMLL <- read.csv(file = paste(tablesDirectory,"enrichment_repeats_hyper_DMRs_non_MLL.txt", sep = ""), sep = "\t")
hyper_nonMLL$userSet <- "nonMLL_hyper"
hypo_nonMLL <- read.csv(file = paste(tablesDirectory,"enrichment_repeats_hypo_DMRs_non_MLL.txt", sep = ""), sep = "\t")
hypo_nonMLL$userSet <- "nonMLL_hypo"

# Load the tables:
hyper <- rbind(hyper_AF4, hyper_AF9, hyper_nonMLL)
hypo <- rbind(hypo_AF4, hypo_AF9, hypo_nonMLL)

# Rename the samples and annotations:
hyper$annotation <- NA
hyper$Sample <- NA
hyper$annotation[which(hyper$filename == "DNAhAT.txt")] <- "DNAhAT"
hyper$annotation[which(hyper$filename == "DNATcMar.txt")] <- "DNATcMar"
hyper$annotation[which(hyper$filename == "LINECR1.txt")] <- "LINECR1"
hyper$annotation[which(hyper$filename == "LINEL1.txt")] <- "LINEL1"
hyper$annotation[which(hyper$filename == "LINEL2.txt")] <- "LINEL2"
hyper$annotation[which(hyper$filename == "Low.txt")] <- "Low"
hyper$annotation[which(hyper$filename == "LTRERV1.txt")] <- "LTRERV1"
hyper$annotation[which(hyper$filename == "LTRERVL.txt")] <- "LTRERVL"
hyper$annotation[which(hyper$filename == "LTRERVLMaLR.txt")] <- "LTRERVLMaLR"
hyper$annotation[which(hyper$filename == "Satellite.txt")] <- "Satellite"
hyper$annotation[which(hyper$filename == "Simple.txt")] <- "Simple"
hyper$annotation[which(hyper$filename == "SINEALU.txt")] <- "SINEALU"
hyper$annotation[which(hyper$filename == "SINEMIR.txt")] <- "SINEMIR"

hypo$annotation <- NA
hypo$Sample <- NA
hypo$annotation[which(hypo$filename == "DNAhAT.txt")] <- "DNAhAT"
hypo$annotation[which(hypo$filename == "DNATcMar.txt")] <- "DNATcMar"
hypo$annotation[which(hypo$filename == "LINECR1.txt")] <- "LINECR1"
hypo$annotation[which(hypo$filename == "LINEL1.txt")] <- "LINEL1"
hypo$annotation[which(hypo$filename == "LINEL2.txt")] <- "LINEL2"
hypo$annotation[which(hypo$filename == "Low.txt")] <- "Low"
hypo$annotation[which(hypo$filename == "LTRERV1.txt")] <- "LTRERV1"
hypo$annotation[which(hypo$filename == "LTRERVL.txt")] <- "LTRERVL"
hypo$annotation[which(hypo$filename == "LTRERVLMaLR.txt")] <- "LTRERVLMaLR"
hypo$annotation[which(hypo$filename == "Satellite.txt")] <- "Satellite"
hypo$annotation[which(hypo$filename == "Simple.txt")] <- "Simple"
hypo$annotation[which(hypo$filename == "SINEALU.txt")] <- "SINEALU"
hypo$annotation[which(hypo$filename == "SINEMIR.txt")] <- "SINEMIR"

hyper <- hyper[!is.na(hyper$annotation),]
hypo <- hypo[!is.na(hypo$annotation),]

# Order the tables:
hyper <- hyper[order(hyper[,"userSet"], hyper[,"annotation"]),]
hypo <- hypo[order(hypo[,"userSet"], hypo[,"annotation"]),]

# Tune up the tables and filter for significant p-values:
hyperfiltered <- hyper
hyperfiltered$oddsRatio[which(hyperfiltered$qValue >= 0.05)] <- 1
hyperfiltered$qValue <- -log10(hyperfiltered$qValue)
hyperfiltered$logOddsRatio <- log2(hyperfiltered$oddsRatio)
hyperfiltered$annotation <- as.factor(hyperfiltered$annotation)
hyperfiltered <- acast(hyperfiltered, annotation ~ userSet, value.var = "logOddsRatio")
hyperfiltered <- hyperfiltered[c("DNAhAT", "DNATcMar", "LINECR1", "LINEL1", "LINEL2", "LTRERV1", "LTRERVL", "LTRERVLMaLR", "Satellite", "SINEALU", "SINEMIR"),]

hypofiltered <- hypo
hypofiltered$oddsRatio[which(hypofiltered$qValue >= 0.05)] <- 1
hypofiltered$qValue <- -log10(hypofiltered$qValue)
hypofiltered$logOddsRatio <- log2(hypofiltered$oddsRatio)
hypofiltered$annotation <- as.factor(hypofiltered$annotation)
hypofiltered <- acast(hypofiltered, annotation ~ userSet, value.var = "logOddsRatio")
hypofiltered <- hypofiltered[c("DNAhAT", "DNATcMar", "LINECR1", "LINEL1", "LINEL2", "LTRERV1", "LTRERVL", "LTRERVLMaLR", "Satellite", "SINEALU", "SINEMIR"),]

hyperhipofiltered <- rbind(hyper, hypo)
hyperhipofiltered$oddsRatio[which(hyperhipofiltered$qValue >= 0.05)] <- 1
hyperhipofiltered$qValue <- -log10(hyperhipofiltered$qValue)
hyperhipofiltered$logOddsRatio <- log2(hyperhipofiltered$oddsRatio)
hyperhipofiltered$annotation <- as.factor(hyperhipofiltered$annotation)
hyperhipofiltered <- acast(hyperhipofiltered, annotation ~ userSet, value.var = "logOddsRatio")
hyperhipofiltered <- hyperhipofiltered[,c("AF4_hyper", "AF9_hyper", "nonMLL_hyper", "AF4_hypo", "AF9_hypo", "nonMLL_hypo")]
hyperhipofiltered <- hyperhipofiltered[c("DNAhAT", "DNATcMar", "LINECR1", "LINEL1", "LINEL2", "LTRERV1", "LTRERVL", "LTRERVLMaLR", "Satellite", "SINEALU", "SINEMIR"),]

# Generate the dataframes for representation:
pdf(file = paste(resultsDirectory, "Enrichments_repeats_hg19_leukemias.pdf", sep = ""), paper = "a4r", height = 21, width = 28, onefile = T)
pheatmap(as.matrix(hyperfiltered), scale = "none", color = colorRampPalette((brewer.pal(n = 7, name ="Blues")))(14), border_color = "#FFFFFF", cluster_rows = F, cluster_cols = F, clustering_method = "ward.D2", fontsize = 5, main = "Heatmap Enrichment OR repeats hyper DMRs", fontsize_row = 5, cellwidth = 10, cellheight = 10, fontsize_col = 5)    
pheatmap(as.matrix(hypofiltered), scale = "none", color = colorRampPalette((brewer.pal(n = 7, name ="Blues")))(14), border_color = "#FFFFFF", cluster_rows = F, cluster_cols = F, clustering_method = "ward.D2", fontsize = 5, main = "Heatmap Enrichment OR repeats hypo DMRs", fontsize_row = 5, cellwidth = 10, cellheight = 10, fontsize_col = 5)    
pheatmap(as.matrix(hyperhipofiltered), scale = "none", color = colorRampPalette((brewer.pal(n = 7, name ="Blues")))(14), border_color = "#FFFFFF", cluster_rows = F, cluster_cols = F, clustering_method = "ward.D2", fontsize = 5, main = "Heatmap Enrichment OR repeats DMRs", fontsize_row = 5, cellwidth = 10, cellheight = 10, fontsize_col = 5)    
dev.off()
