#############################################################################
### PROJECT: Differential methylation analysis of Infant leukemias
### and differentiated blood cells with 850k arrays
###
### Script for the analysis of Methylation array data (850K)
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

# Download and install the required packages:
#install.packages("quadprog")
#source("http://bioconductor.org/biocLite.R")
#biocLite("limma")
#biocLite("minfi")
#biocLite("IlluminaHumanMethylationEPICmanifest")
#biocLite("IlluminaHumanMethylation450kmanifest")
#biocLite("RColorBrewer")
#biocLite("missMethyl") 
#biocLite("matrixStats")
#biocLite("Gviz") 
#biocLite("DMRcate") 
#biocLite("stringr")
#biocLite("IlluminaHumanMethylationEPICanno.ilm10b2.hg19")
#biocLite("IlluminaHumanMethylation450kanno.ilmn12.hg19")
#biocLite("GO.db")
#biocLite("FlowSorted.Blood.450k")
#biocLite("genefilter")
#biocLite("ChAMP")
#biocLite("sva")
#install.packages("FactoMineR")
#install.packages("devtools")
#install.packages("gplots")
#install.packages("cluster")
#install.packages("gridExtra")
#devtools::install_github("kassambara/factoextra")
#biocLite("ENmix")

# Load the required packages:
library("RColorBrewer")
library("pheatmap")
library("genefilter")
library(tidyr)
library(dplyr)
library(sva)
library(RColorBrewer)
library(pheatmap)
library(matrixStats)
library(minfi)
library(limma)
library(IlluminaHumanMethylation450kanno.ilmn12.hg19)
library(DMRcate)
library(missMethyl)
library(Gviz)
library(DMRcate)

options("width"=110) #  using R in terminal mode on Linux, making easier the inspection of data sets
###################################################################
#####                       READING DATA                      #####
###################################################################

# Set up the directory for the results:
resultsDirectory <- ""
tablesDirectory <- ""

# Load the data:
phenoData <- read.csv(file = paste(tablesDirectory, "phenoData.csv", sep = "/"), header = T, sep = ",", stringsAsFactors = F)
rownames(phenoData) <- phenoData$CODE
bValues <- read.table(file = paste(tablesDirectory, "bVals.txt", sep = "/"), header = T, sep = "\t")
colnames(bValues) <- phenoData$CODE
mValues <- read.table(file = paste(tablesDirectory, "mVals.txt", sep = "/"), header = T, sep = "\t")
colnames(mValues) <- phenoData$CODE

# Select Progenitors and B cells:
phenoData_filtered <- phenoData[which(phenoData$Group == "B-PROGENITORS" | phenoData$Group == "MLL-AF4"),]
bVals_filtered <- bValues[,phenoData_filtered$CODE]
mVals_filtered <- mValues[,phenoData_filtered$CODE]

# Set the color palette for visual representations:
pal <- brewer.pal(2, "Set1")


###################################################################
#####           BATCH EFFECT CORRECTION: SVA                  #####
###################################################################

mod = model.matrix(~as.factor(Group), data=phenoData_filtered)
colnames(mod) <- c("Intercept", "cancer_normal")
mod0 = model.matrix(~1,data=phenoData_filtered)

n.sv = num.sv(as.matrix(mVals_filtered),mod,method="leek")
svobj = sva(as.matrix(mVals_filtered),mod,mod0,n.sv=n.sv)
colnames(svobj$sv) <- paste(rep("col",ncol(svobj$sv)),c(1:ncol(svobj$sv)),sep="")

modSv = cbind(mod,svobj$sv)
mod0Sv = cbind(mod0,svobj$sv)


###################################################################################
#####      PROBE-WISE DIFFERENTIAL METHYLATION ANALYSIS (SVA CORRECTION)      #####
###################################################################################

# Fit the linear model:
fit <- lmFit(mVals_filtered, modSv)
cancer_normal <- factor(phenoData_filtered$Group)

# Create a contrast matrix for specific comparisons:
contMatrix <- makeContrasts(cancer_normal, levels = modSv)

# Fit the contrasts:
fit2 <- contrasts.fit(fit, contMatrix)
fit2 <- eBayes(fit2)

# Look at the numbers of Differentially Methylated CpGs at FDR < 0.05:
summary(decideTests(fit2))

# Get the table of results for the first contrast:
DMPs_filtered <- limma::topTable(fit2, number = Inf, coef = 1, p.value = 0.05)
DMPs_filtered$Methylstat <- NA
DMPs_filtered$Methylstat[which(DMPs_filtered$logFC > 0)] <- "hyper"
DMPs_filtered$Methylstat[which(DMPs_filtered$logFC < 0)] <- "hypo"
bValues2 <- bVals_filtered[rownames(DMPs_filtered),]
bValues2$diff <- rowMeans(bValues2[,1:37]) - rowMeans(bValues2[,38:43])  ### Check out every time
bValues2 <- bValues2[which(abs(bValues2$diff) > 0.25),]
DMPs_filtered <- DMPs_filtered[rownames(bValues2),]

# Write the resulting dataframe in a CSV file:
write.table(DMPs_filtered, file = paste(tablesDirectory, "DMPs_filtered_AF4.txt", sep = "/"), sep = "\t", row.names = TRUE)

# Plot the top 4 most differentially methylated CpGs:
pdf(file= "topprobes_AF4.pdf", paper="a4r", height=21, width=28, onefile=TRUE)
par(mfrow=c(2,5))
sapply(rownames(DMPs_filtered)[1:10], function(cpg){plotCpg(as.matrix(bVals_filtered), cpg = cpg, pheno = cancer_normal)})
bVals <- bVals_filtered[rownames(DMPs_filtered),]
annotation_cancer_normal <- data.frame(cancergroup = as.character(phenoData_filtered$Group))
rownames(annotation_cancer_normal) <- colnames(bVals_filtered)
pheatmap(bVals[1:10000,], cluster_rows = T, cluster_cols = T, color = colorRampPalette((brewer.pal(n = 7, name = "Reds")))(100), annotation = annotation_cancer_normal, clustering_method = "ward.D2", main = "Heatmap BVals 10000 top significant probes AF4")

dev.off()
