#############################################################################
### PROJECT: Differential methylation analysis of Infant leukemias
### and differentiated blood cells with 850k arrays
###
### Script for the analysis of Methylation array data (850K)
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

# Download and install the required packages:
#install.packages("quadprog")
#source("http://bioconductor.org/biocLite.R")
#biocLite("limma")
#biocLite("minfi")
#biocLite("IlluminaHumanMethylationEPICmanifest")
#biocLite("IlluminaHumanMethylation450kmanifest")
#biocLite("RColorBrewer")
#biocLite("missMethyl") 
#biocLite("matrixStats")
#biocLite("Gviz") 
#biocLite("DMRcate") 
#biocLite("stringr")
#biocLite("IlluminaHumanMethylationEPICanno.ilm10b2.hg19")
#biocLite("IlluminaHumanMethylation450kanno.ilmn12.hg19")
#biocLite("GO.db")
#biocLite("FlowSorted.Blood.450k")
#biocLite("genefilter")
#biocLite("ChAMP")
#biocLite("sva")
#install.packages("FactoMineR")
#install.packages("devtools")
#install.packages("gplots")
#install.packages("cluster")
#install.packages("gridExtra")
#devtools::install_github("kassambara/factoextra")
#biocLite("ENmix")

# Load the required packages:
library("RColorBrewer")
library("pheatmap")
library("genefilter")

options("width"=110) #  using R in terminal mode on Linux, making easier the inspection of data sets
###################################################################
#####                       READING DATA                      #####
###################################################################

# Set up the directory for the results:
resultsDirectory <- ""
tablesDirectory <- ""

# Load the data:
phenoData <- read.csv(file = paste(tablesDirectory, "phenoData.csv", sep = "/"), header = T, sep = ",", stringsAsFactors = F)
rownames(phenoData) <- phenoData$CODE
bVals <- read.table(file = paste(tablesDirectory, "bVals.txt", sep = "/"), header = T, sep = "\t")
colnames(bVals) <- rownames(phenoData)
#write.table(bVals, file = "bVals.txt", sep = "\t", col.names = TRUE, row.names = TRUE, quote = F)

# Set the color palette for visual representations:
groups <- factor(phenoData$Group, levels = c("B-PROGENITORS", "MLL-AF4", "MLL-AF9", "MLL GERMLINE", "B_Lymphocytes_CD19"))
pal <- factor(phenoData$color, levels = c("#3288BD", "#D53E4F", "#2eaa6c", "#e8db4c", "#5e4fa2"))

# Select Progenitors and B cells:
phenoData_filtered <- phenoData[which(phenoData$Group == "B-PROGENITORS" | phenoData$Group == "B_Lymphocytes_CD19"),]
bVals_filtered <- bVals[,phenoData_filtered$CODE]

bVals_filtered$mean_progenitors <- rowMeans(bVals_filtered[,c(2:7)])
bVals_filtered$diff_B_progenitors <- bVals_filtered$`8` - bVals_filtered$mean_progenitors
bVals_filtered <- bVals_filtered[which(bVals_filtered$diff_B_progenitors >= 0.25 | bVals_filtered$diff_B_progenitors <= -0.25),]

###################################################################
#####                    Scatterplots                         #####
###################################################################

# Generate scatterplot to inspect differences between ESCs and the rest of conditions:

pdf(file=paste(resultsDirectory, "Scatterplots_significant_025_diffB_vs_proB.pdf", sep = "/"), paper="a4r", height=21, width=28, onefile=TRUE)
smoothScatter(x=bVals_filtered[,1], y=bVals_filtered[,8], xlab="Beta values Bcells", ylab="Beta values ProB", main="smooth scatter plot B vs ProB")
abline(v = 0.25, untf = FALSE, col = "darkgrey", lty = 2); abline(v = 0.75, untf = FALSE, col = "darkgrey", lty = 2); abline(h = 0.25, untf = FALSE, col = "darkgrey", lty = 2); abline(h = 0.75, untf = FALSE, col = "darkgrey", lty = 2)
text(0.9, 0.9, paste("n = ", nrow(bVals_filtered), sep = ""))
dev.off()

###################################################################
#####                       Heatmap                           #####
###################################################################

# Plot the heatmap:
pdf(file=paste(resultsDirectory, "Heatmap_20000_most_variable__significant_probes.pdf", sep = "/"), paper="a4r", height=21, width=28, onefile=TRUE)
bVals_filtered2 <- bVals_filtered[head(order(-rowVars(as.matrix(bVals_filtered[,c(1:7)]))), 20000),]
annotation_control_case <- data.frame(group = as.character(phenoData_filtered$Group))
rownames(annotation_control_case) <- colnames(bVals_filtered2)[1:7]
pheatmap(bVals_filtered2[, c(1:7)], cluster_rows = T, cluster_cols = T, color = colorRampPalette((brewer.pal(n = 7, name = "Blues")))(100), annotation = annotation_control_case, clustering_method = "ward.D2", main = "Heatmap BVals 20000 most variable significant probes")
dev.off()

# Write the tables with the results:
setwd(tablesDirectory)
write.table(bVals_filtered, file = "DMPs_filtered_Bcells.txt", sep = "\t", col.names = TRUE, row.names = TRUE, quote = F)
