#############################################################################
### PROJECT: Differential methylation analysis of Infant leukemias
### and differentiated blood cells with 850k arrays
###
### Script for the analysis of Methylation array data (850K)
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

# Load the required packages:
library("plyr")
library("dplyr")
library("tidyr")
library("ggplot2")
library("reshape2")
library("pheatmap")
library("RColorBrewer")
library("scales")
library("extrafont")

tablesDirectory <- ""
phenoDirectory <- ""
resultsDirectory <- ""

# Load the tables:
hyper_COMMON <- read.csv(file = paste(tablesDirectory, "enrichment_GTRD_tfbs_hyper_COMMON_leukemia.txt", sep = ""), sep = "\t", stringsAsFactors = F)
hyper_AF4 <- read.csv(file = paste(tablesDirectory, "enrichment_GTRD_tfbs_hyper_AF4_leukemia.txt", sep = ""), sep = "\t", stringsAsFactors = F)
hyper_AF9 <- read.csv(file = paste(tablesDirectory, "enrichment_GTRD_tfbs_hyper_AF9_leukemia.txt", sep = ""), sep = "\t", stringsAsFactors = F)
hyper_GERM <- read.csv(file = paste(tablesDirectory, "enrichment_GTRD_tfbs_hyper_GERM_leukemia.txt", sep = ""), sep = "\t", stringsAsFactors = F)
hyper_B <- read.csv(file = paste(tablesDirectory, "enrichment_GTRD_tfbs_hyper_B_leukemia.txt", sep = ""), sep = "\t", stringsAsFactors = F)
hypo_COMMON <- read.csv(file = paste(tablesDirectory, "enrichment_GTRD_tfbs_hypo_COMMON_leukemia.txt", sep = ""), sep = "\t", stringsAsFactors = F)
hypo_AF4 <- read.csv(file = paste(tablesDirectory, "enrichment_GTRD_tfbs_hypo_AF4_leukemia.txt", sep = ""), sep = "\t", stringsAsFactors = F)
hypo_AF9 <- read.csv(file = paste(tablesDirectory, "enrichment_GTRD_tfbs_hypo_AF9_leukemia.txt", sep = ""), sep = "\t", stringsAsFactors = F)
hypo_GERM <- read.csv(file = paste(tablesDirectory, "enrichment_GTRD_tfbs_hypo_GERM_leukemia.txt", sep = ""), sep = "\t", stringsAsFactors = F)
hypo_B <- read.csv(file = paste(tablesDirectory, "enrichment_GTRD_tfbs_hypo_B_leukemia.txt", sep = ""), sep = "\t", stringsAsFactors = F)

hyper_COMMON$Dataset <- "hyper_COMMON"
hyper_AF4$Dataset <- "hyper_AF4"
hyper_AF9$Dataset <- "hyper_AF9"
hyper_GERM$Dataset <- "hyper_GERM"
hyper_B$Dataset <- "hyper_B"
hypo_COMMON$Dataset <- "hypo_COMMON"
hypo_AF4$Dataset <- "hypo_AF4"
hypo_AF9$Dataset <- "hypo_AF9"
hypo_GERM$Dataset <- "hypo_GERM"
hypo_B$Dataset <- "hypo_B"

pheno <- read.csv(file = paste(phenoDirectory, "index.txt", sep = ""), sep = "\t")
pheno <- pheno[order(pheno[,"antibody"]),]

megatable_hyper <- rbind(hyper_COMMON, hyper_AF4, hyper_AF9, hyper_GERM, hyper_B)
megatable_hypo <- rbind(hypo_COMMON, hypo_AF4, hypo_AF9, hypo_GERM, hypo_B)


# Keep the desired antibodies:
TFlist <- c("ZEB1", "ZBTB33", "USF-2", "USF-1", "TCF-7", "Tal-1", "THAP1", "STAT6", "STAT4", "STAT3", "STAT1", "Spi-B", "Sp4", "Sp3", "Sp2", "Sp1", 
            "SMAD4", "SMAD3", "SMAD2", "SMAD1", "RXR_1", "RXR_2", "Runx3", "Runx2", "Runx1", "REST", "RAR-_1", "PU.1", "PLAG1", "NRF-1", 
            "NF-YA", "NF-Bp105", "NF-Bp100", "NF-Bp65", "NF-1C", "MLL", "Meis1", "MEF-2B", "KLF15", "KLF9", "KLF6", "KLF5", "KLF4", "KLF3", 
            "KLF1", "JunD", "JunB", "IRF5", "IRF4", "IRF3", "IRF2", "IRF1", "Hox-D13", "Hox-D12", "Hox-D11", "Hox-C9", "Hox-B4", "Hox-A13", 
            "Hox-A11", "Hox-A10", "Hox-A9", "Hox-A2", "Hox-A1", "GATA-6", "GATA-5", "GATA-4", "GATA-3", "GATA-2", "GATA-1", "GABP_1", "Fra-1",
            "FosB", "Fli-1", "ETV6", "ETV4", "ETV2", "ETV1", "Elk-1", "Elk-4", "Elf-5", "Elf-3", "Elf-1", "EGR2", "EGR1", "EBF1", "E2F-7", "E2F6", "E2F-4", 
            "E2F-3", "E2F-1", "E2A", "c-Myc", "c-Myb", "c-Jun", "c-Fos", "c-Ets-2", "c-Ets-1", "CEBP_1", "BCL11A", "B-ATF", "AP-2_3", "AP-2_1")

megatable_hyper <- megatable_hyper[which(megatable_hyper$antibody %in% TFlist),]
megatable_hyper$antibody <- as.factor(megatable_hyper$antibody)
megatable_hyper$Dataset <- factor(megatable_hyper$Dataset, levels = c("hyper_COMMON", "hyper_AF4", "hyper_AF9", "hyper_GERM", "hyper_B"))
megatable_hypo <- megatable_hypo[which(megatable_hypo$antibody %in% TFlist),]
megatable_hypo$antibody <- as.factor(megatable_hypo$antibody)
megatable_hypo$Dataset <- factor(megatable_hypo$Dataset, levels = c("hypo_COMMON", "hypo_AF4", "hypo_AF9", "hypo_GERM", "hypo_B"))

# Tune up the tables and filter for significant p-values:
hyperfiltered <- megatable_hyper
hyperfiltered$percentage <- (hyperfiltered$support / hyperfiltered$c)*100
hyperfiltered$logOddsRatio[which(hyperfiltered$qValue >= 0.05)] <- 1
hyperfiltered$percentage[which(hyperfiltered$qValue >= 0.05)] <- 0
hypermax <- max(hyperfiltered$pValueLog[is.finite(hyperfiltered$pValueLog)])
hyperfiltered$pValueLog[which(is.infinite(hyperfiltered$pValueLog))] <- hypermax
hyperfiltered$qValue <- -log10(hyperfiltered$qValue)
rownames(hyperfiltered) <- c(1:length(hyperfiltered$userSet))

hypofiltered <- megatable_hypo
hypofiltered$percentage <- (hypofiltered$support / hypofiltered$c)*100
hypofiltered$logOddsRatio[which(hypofiltered$qValue >= 0.05)] <- 1
hypofiltered$percentage[which(hypofiltered$qValue >= 0.05)] <- 0
hypomax <- max(hypofiltered$pValueLog[is.finite(hypofiltered$pValueLog)])
hypofiltered$pValueLog[which(is.infinite(hypofiltered$pValueLog))] <- hypomax
hypofiltered$qValue <- -log10(hypofiltered$qValue)
rownames(hypofiltered) <- c(1:length(hypofiltered$userSet))

hypermaxqval <- max(hyperfiltered$qValue[is.finite(hyperfiltered$qValue)])
hyperfiltered$qValue[which(is.infinite(hyperfiltered$qValue))] <- hypermaxqval
hyperfiltered$qValue[which(hyperfiltered$qValue > 100)] <- 100
hyperfiltered$logOddsRatio[which(hyperfiltered$logOddsRatio > 3.04)] <- 4
hypomaxqval <- max(hypofiltered$qValue[is.finite(hypofiltered$qValue)])
hypofiltered$qValue[which(is.infinite(hypofiltered$qValue))] <- hypomaxqval
hypofiltered$qValue[which(hypofiltered$qValue > 100)] <- 100
hypofiltered$logOddsRatio[which(hypofiltered$logOddsRatio > 3.8)] <- 4

max(hyperfiltered$qValue)
max(hypofiltered$qValue)

# Generate the dataframes form representation:
cairo_pdf(file = paste(resultsDirectory, "bubble_heatmap_enrichments_GTRD_TFBS.pdf", sep = ""), height = 12, width = 3, onefile = T)
ggplot(hyperfiltered, aes(Dataset, antibody)) + geom_point(aes(size=hyperfiltered$logOddsRatio, colour = qValue, fill = qValue), shape = 21) + scale_colour_gradient(low = "#f9ebe8", high = "#8e1700") + scale_fill_gradient(low = "#f9ebe8", high = "#8e1700") + scale_size_identity() + theme_bw() + theme(text=element_text(family="Arial"), axis.text.x = element_text(angle = 90, hjust = 1))
ggplot(hypofiltered, aes(Dataset, antibody)) + geom_point(aes(size=hypofiltered$logOddsRatio, colour = qValue, fill = qValue), shape = 21) + scale_colour_gradient(low = "#e8f9fc", high = "#007f99") + scale_fill_gradient(low = "#e8f9fc", high = "#007f99") + scale_size_identity() + theme_bw() + theme(text=element_text(family="Arial"), axis.text.x = element_text(angle = 90, hjust = 1))
dev.off()
