#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script to plot heatmaps from enrichment analyses
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

#################################################################################
###         Generate heatmaps for visualization (histone marks)               ###
#################################################################################

library(plyr)
library(dplyr)
library(tidyr)
library(ggplot2)
library(reshape2)
library(pheatmap)
library(RColorBrewer)
library(scales)


tablesDirectory <- ""
phenoDirectory <- ""
resultsDirectory <- ""

# Load the tables:
hyper <- read.csv(file = paste(tablesDirectory, "enrichment_roadmap_epigenomics_hyper_B_leukemia.txt", sep = ""), sep = "\t")
hypo <- read.csv(file = paste(tablesDirectory, "enrichment_roadmap_epigenomics_hypo_B_leukemia.txt", sep = ""), sep = "\t")
hyper <- separate(hyper, col = filename, into = c("case", "rest"), remove = F)
hypo <- separate(hypo, col = filename, into = c("case", "rest"), remove = F)
pheno <- read.csv(file = paste(phenoDirectory, "index.txt", sep = ""), sep = "\t")
pheno <- pheno[order(pheno[,"id"], pheno[, "antibody"]),]

# Order the tables:
hyper <- hyper[order(hyper[,"dbSet"], hyper[,"antibody"]),]
hyper2 <- hyper[which(hyper$antibody == "H3K4me1" | hyper$antibody == "H3K4me3" | hyper$antibody == "H3K27ac" | hyper$antibody == "H3K36me3" | hyper$antibody == "H3K27me3" | hyper$antibody == "H3K9me3"),]
hyper2unique <- hyper2$case[which(hyper2$antibody == "H3K27ac")]
hyper2 <- hyper2[which(hyper2$case %in% hyper2unique),]
hyper2 <- hyper2[order(hyper2[,"dbSet"], hyper2[,"antibody"]),]

hypo <- hypo[order(hypo[,"dbSet"], hypo[,"antibody"]),]
hypo2 <- hypo[which(hypo$antibody == "H3K4me1" | hypo$antibody == "H3K4me3" | hypo$antibody == "H3K27ac" | hypo$antibody == "H3K36me3" | hypo$antibody == "H3K27me3" | hypo$antibody == "H3K9me3"),]
hypo2unique <- hypo2$case[which(hypo2$antibody == "H3K27ac")]
hypo2 <- hypo2[which(hypo2$case %in% hypo2unique),]
hypo2 <- hypo2[order(hypo2[,"dbSet"], hypo2[,"antibody"]),]

pheno2 <- pheno[which(pheno$antibody == "H3K4me1" | pheno$antibody == "H3K4me3" | pheno$antibody == "H3K27ac" | pheno$antibody == "H3K36me3" | pheno$antibody == "H3K27me3" | pheno$antibody == "H3K9me3"),]
pheno2unique <- pheno2$donor[which(pheno2$antibody == "H3K27ac")]
pheno2 <- pheno2[which(pheno2$donor %in% pheno2unique),]
pheno2 <- pheno2[order(pheno2[,"id"], pheno2[,"antibody"]),]

hyper2$cellType <- pheno2$mnemonic
hypo2$cellType <- pheno2$mnemonic

orderhistonemarks <- c("H3K4me1", "H3K4me3", "H3K27ac", "H3K36me3", "H3K27me3", "H3K9me3")

# Tune up the tables and filter for significant p-values:
hyperfiltered <- hyper2
hyperfiltered$logOddsRatio[which(hyperfiltered$pValueLog <= 1.30)] <- 1

hypofiltered <- hypo2
hypofiltered$logOddsRatio[which(hypofiltered$pValueLog <= 1.30)] <- 1

# Generate the dataframe from the table:
hyperfiltered2 <- dcast(hyperfiltered, cellType ~ antibody, value.var = "logOddsRatio")
hypofiltered2 <- dcast(hypofiltered, cellType ~ antibody, value.var = "logOddsRatio")

rownames(hyperfiltered2) <- hyperfiltered2$cellType
hyperfiltered2 <- hyperfiltered2[,-1]
rownames(hypofiltered2) <- hypofiltered2$cellType
hypofiltered2 <- hypofiltered2[,-1]

hyperfiltered2 <- hyperfiltered2[,orderhistonemarks]
hypofiltered2 <- hypofiltered2[,orderhistonemarks]

hyperfiltered3 <- hyperfiltered2
hypofiltered3 <- hypofiltered2

hyperfiltered3$cellType <- as.character(rownames(hyperfiltered3))
hypofiltered3$cellType <- as.character(rownames(hypofiltered3))

hyperfiltered3 <- melt(hyperfiltered3)
hypofiltered3 <- melt(hypofiltered3)

hyperfiltered2[is.na(hyperfiltered2)] <- 1
hypofiltered2[is.na(hypofiltered2)] <- 1

hyperfiltered_encode <- hyperfiltered2[c("BLD.MOB.CD34.PC.F", "BLD.CD19.PPC", "BLD.GM12878", "BLD.DND41.CNCR", "BLD.K562.CNCR"),]
hypofiltered_encode <- hypofiltered2[c("BLD.MOB.CD34.PC.F", "BLD.CD19.PPC", "BLD.GM12878", "BLD.DND41.CNCR", "BLD.K562.CNCR"),]

# Generate the heatmaps from the table:
annotation_tissue <- data.frame(group = as.character(pheno2$anatomy[seq(1, length(pheno2$anatomy), 6)]))
rownames(annotation_tissue) <- unique(hyperfiltered$cellType)

annotation_tissue2 <- data.frame(group = c("HSC", "LB", "LYMPHOBLAST", "LEUKEMIA", "LEUKEMIA"))
rownames(annotation_tissue2) <- rownames(hyperfiltered_encode)

pdf(file = paste(resultsDirectory, "Heatmaps_enrichments_histone_marks_B_leukemia.pdf", sep = ""), paper = "a4r", height = 21, width = 28, onefile = T)

pheatmap(as.matrix(hyperfiltered_encode), scale = "none", color = colorRampPalette(c("#EFEFF0","#5e4fa2"))(100), border_color = "#FFFFFF", cluster_rows = F, cluster_cols = F, annotation_row = annotation_tissue2, clustering_method = "ward.D2", main = "Heatmap Enrichment OR histone marks hyper CpGs B", fontsize_row = 5, cellwidth = 10, cellheight = 5)  
pheatmap(as.matrix(hypofiltered_encode), scale = "none", color = colorRampPalette(c("#EFEFF0","#5e4fa2"))(100), border_color = "#FFFFFF", cluster_rows = F, cluster_cols = F, annotation_row = annotation_tissue2, clustering_method = "ward.D2", main = "Heatmap Enrichment OR histone marks hypo CpGs B", fontsize_row = 5, cellwidth = 10, cellheight = 5)  

dev.off()
