#############################################################################
### PROJECT: Differential methylation analysis of Infant leukemias
### and differentiated blood cells with 850k arrays
###
### Script for the analysis of Methylation array data (850K)
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

library(magrittr)
library(dplyr)
library(tidyr)
library(ggplot2)
library(ChIPseeker)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(clusterProfiler)
library(ReactomePA)
library(VennDiagram)
library(reshape2)
library(pheatmap)
library(RColorBrewer)
library(LOLA)

tablesDirectory <- ""

# Load the datasets to establish to plot the pie charts:
# Run this script for each dataset (AF4, AF9, nonMLL)
txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
setwd(tablesDirectory)
allprobes_cpg <- read.csv(file = "./annotation_EPIC.txt", sep = "\t", stringsAsFactors = F)
colnames(allprobes_cpg)[2] <- "start"
hypermethylation <- read.csv(file = "./annotation_GERM_hyper_probes.txt", sep = "\t", stringsAsFactors = F)
hypomethylation <- read.csv(file = "./annotation_GERM_hypo_probes.txt", sep = "\t", stringsAsFactors = F)

# Generate Granges files for the hyper and the hypomethylated probes:
hypermethylationprobes <- allprobes_cpg[which(allprobes_cpg$Name %in% hypermethylation$Name),]
hypermethylationprobes <- hypermethylationprobes[order(hypermethylationprobes[,"chr"], hypermethylationprobes[,"start"]),]
regionSethyper <- makeGRangesFromDataFrame(hypermethylationprobes, ignore.strand = TRUE, seqnames.field = "chr", start.field = "start", end.field = "end")
hypomethylationprobes <- allprobes_cpg[which(allprobes_cpg$Name %in% hypomethylation$Name),]
hypomethylationprobes <- hypomethylationprobes[order(hypomethylationprobes[,"chr"], hypomethylationprobes[,"start"]),]
regionSethypo <- makeGRangesFromDataFrame(hypomethylationprobes, ignore.strand = TRUE, seqnames.field = "chr", start.field = "start", end.field = "end")
allprobes_cpg <- allprobes_cpg[order(allprobes_cpg[,"chr"], allprobes_cpg[,"start"]),]
regionSetAll = makeGRangesFromDataFrame(allprobes_cpg, ignore.strand = TRUE, seqnames.field = "chr", start.field = "start", end.field = "end")

# # Load the databases with the Regions of Interest:
# regionDB_1 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state1_TssA")
# regionDB_2 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state2_TssFlnk")
# regionDB_3 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state3_TssFlnkU")
# regionDB_4 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state4_TssFlnkD")
# regionDB_5 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state5_Tx")
# regionDB_6 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state6_TxWk")
# regionDB_7 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state7_EnhG1")
# regionDB_8 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state8_EnhG2")
# regionDB_9 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state9_EnhA1")
# regionDB_10 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state10_EnhA2")
# regionDB_11 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state11_EnhWk")
# regionDB_12 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state12_ZNF_Rpts")
# regionDB_13 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state13_Het")
# regionDB_14 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state14_TssBiv")
# regionDB_15 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state15_EnhBiv")
# regionDB_16 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state16_ReprPC")
# regionDB_17 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state17_ReprPCWk")
# regionDB_18 <- loadRegionDB(dbLocation = "~/Databases/regionDB/hg19/SEGMENTATION_files_roadmap_encodeplusroadmap18states/", collections = "chromHMM_state18_Quies")

# Generate tables from lists (probes)
userSets = GRangesList(hyper = regionSethyper, hypo = regionSethypo)

locResults_hyper1 = runLOLA(userSets[1], regionSetAll, regionDB_1, cores=4)
locResults_hyper2 = runLOLA(userSets[1], regionSetAll, regionDB_2, cores=4)
locResults_hyper3 = runLOLA(userSets[1], regionSetAll, regionDB_3, cores=4)
locResults_hyper4 = runLOLA(userSets[1], regionSetAll, regionDB_4, cores=4)
locResults_hyper5 = runLOLA(userSets[1], regionSetAll, regionDB_5, cores=4)
locResults_hyper6 = runLOLA(userSets[1], regionSetAll, regionDB_6, cores=4)
locResults_hyper7 = runLOLA(userSets[1], regionSetAll, regionDB_7, cores=4)
locResults_hyper8 = runLOLA(userSets[1], regionSetAll, regionDB_8, cores=4)
locResults_hyper9 = runLOLA(userSets[1], regionSetAll, regionDB_9, cores=4)
locResults_hyper10 = runLOLA(userSets[1], regionSetAll, regionDB_10, cores=4)
locResults_hyper11 = runLOLA(userSets[1], regionSetAll, regionDB_11, cores=4)
locResults_hyper12 = runLOLA(userSets[1], regionSetAll, regionDB_12, cores=4)
locResults_hyper13 = runLOLA(userSets[1], regionSetAll, regionDB_13, cores=4)
locResults_hyper14 = runLOLA(userSets[1], regionSetAll, regionDB_14, cores=4)
locResults_hyper15 = runLOLA(userSets[1], regionSetAll, regionDB_15, cores=4)
locResults_hyper16 = runLOLA(userSets[1], regionSetAll, regionDB_16, cores=4)
locResults_hyper17 = runLOLA(userSets[1], regionSetAll, regionDB_17, cores=4)
locResults_hyper18 = runLOLA(userSets[1], regionSetAll, regionDB_18, cores=4)

locResults_hypo1 = runLOLA(userSets[2], regionSetAll, regionDB_1, cores=4)
locResults_hypo2 = runLOLA(userSets[2], regionSetAll, regionDB_2, cores=4)
locResults_hypo3 = runLOLA(userSets[2], regionSetAll, regionDB_3, cores=4)
locResults_hypo4 = runLOLA(userSets[2], regionSetAll, regionDB_4, cores=4)
locResults_hypo5 = runLOLA(userSets[2], regionSetAll, regionDB_5, cores=4)
locResults_hypo6 = runLOLA(userSets[2], regionSetAll, regionDB_6, cores=4)
locResults_hypo7 = runLOLA(userSets[2], regionSetAll, regionDB_7, cores=4)
locResults_hypo8 = runLOLA(userSets[2], regionSetAll, regionDB_8, cores=4)
locResults_hypo9 = runLOLA(userSets[2], regionSetAll, regionDB_9, cores=4)
locResults_hypo10 = runLOLA(userSets[2], regionSetAll, regionDB_10, cores=4)
locResults_hypo11 = runLOLA(userSets[2], regionSetAll, regionDB_11, cores=4)
locResults_hypo12 = runLOLA(userSets[2], regionSetAll, regionDB_12, cores=4)
locResults_hypo13 = runLOLA(userSets[2], regionSetAll, regionDB_13, cores=4)
locResults_hypo14 = runLOLA(userSets[2], regionSetAll, regionDB_14, cores=4)
locResults_hypo15 = runLOLA(userSets[2], regionSetAll, regionDB_15, cores=4)
locResults_hypo16 = runLOLA(userSets[2], regionSetAll, regionDB_16, cores=4)
locResults_hypo17 = runLOLA(userSets[2], regionSetAll, regionDB_17, cores=4)
locResults_hypo18 = runLOLA(userSets[2], regionSetAll, regionDB_18, cores=4)

# Save the results in a R object:
save(locResults_hyper1, locResults_hyper2, locResults_hyper3, locResults_hyper4, locResults_hyper5, locResults_hyper6, locResults_hyper7, locResults_hyper8, locResults_hyper9, locResults_hyper10, locResults_hyper11, locResults_hyper12, locResults_hyper13, locResults_hyper14, locResults_hyper15, locResults_hyper16, locResults_hyper17, locResults_hyper18, file = paste(tablesDirectory,"enrichment_roadmap_epigenomics_hyper_GERM_leukemia.RData", sep = "/"))
save(locResults_hypo1, locResults_hypo2, locResults_hypo3, locResults_hypo4, locResults_hypo5, locResults_hypo6, locResults_hypo7, locResults_hypo8, locResults_hypo9, locResults_hypo10, locResults_hypo11, locResults_hypo12, locResults_hypo13, locResults_hypo14, locResults_hypo15, locResults_hypo16, locResults_hypo17, locResults_hypo18, file = paste(tablesDirectory,"enrichment_roadmap_epigenomics_hypo_GERM_leukemia.RData", sep = "/"))

# Generate the dataframe with the pvalues and OR:
locResults_hyper <- rbind(locResults_hyper1, locResults_hyper2, locResults_hyper3, locResults_hyper4, locResults_hyper5, locResults_hyper6, locResults_hyper7, locResults_hyper8, locResults_hyper9, locResults_hyper10, locResults_hyper11, locResults_hyper12, locResults_hyper13, locResults_hyper14, locResults_hyper15, locResults_hyper16, locResults_hyper17, locResults_hyper18, fill = TRUE)
locResults_hypo <- rbind(locResults_hypo1, locResults_hypo2, locResults_hypo3, locResults_hypo4, locResults_hypo5, locResults_hypo6, locResults_hypo7, locResults_hypo8, locResults_hypo9, locResults_hypo10, locResults_hypo11, locResults_hypo12, locResults_hypo13, locResults_hypo14, locResults_hypo15, locResults_hypo16, locResults_hypo17, locResults_hypo18, fill = TRUE)

# Save the results in a R table:
write.table(locResults_hyper, file = paste(tablesDirectory,"enrichment_roadmap_segmentation_hyper_GERM_leukemia.txt", sep = "/"), quote = F, col.names = T, row.names = F, sep = "\t")
write.table(locResults_hypo, file = paste(tablesDirectory,"enrichment_roadmap_segmentation_hypo_GERM_leukemia.txt", sep = "/"), quote = F, col.names = T, row.names = F, sep = "\t")

