#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script for the generation of heatmaps from enrichment data
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

#################################################################################
###         Generate heatmaps for visualization (chromatin states)            ###
#################################################################################

library(plyr)
library(dplyr)
library(tidyr)
library(ggplot2)
library(reshape2)
library(pheatmap)
library(RColorBrewer)
library(scales)


tablesDirectory <- ""
phenoDirectory <- ""
resultsDirectory <- ""

# Load the tables:
hyper <- read.csv(file = paste(tablesDirectory, "enrichment_roadmap_segmentation_hyper_GERM_leukemia.txt", sep = ""), sep = "\t")
hypo <- read.csv(file = paste(tablesDirectory, "enrichment_roadmap_segmentation_hypo_GERM_leukemia.txt", sep = ""), sep = "\t")
pheno <- read.csv(file = paste(phenoDirectory, "index.txt", sep = ""), sep = "\t")
pheno <- pheno[order(pheno[,"anatomy"]),]

# Order the tables:
hyper <- hyper[order(hyper[,"dbSet"], hyper[,"cellType"]),]
hypo <- hypo[order(hypo[,"dbSet"], hypo[,"cellType"]),]
orderstates <- c("chromHMM_state1_TssA", "chromHMM_state2_TssFlnk", "chromHMM_state3_TssFlnkU", "chromHMM_state4_TssFlnkD", "chromHMM_state5_Tx", "chromHMM_state6_TxWk", "chromHMM_state7_EnhG1", "chromHMM_state8_EnhG2", "chromHMM_state9_EnhA1", "chromHMM_state10_EnhA2", "chromHMM_state11_EnhWk", "chromHMM_state12_ZNF_Rpts", "chromHMM_state13_Het", "chromHMM_state14_TssBiv", "chromHMM_state15_EnhBiv", "chromHMM_state16_ReprPC", "chromHMM_state17_ReprPCWk", "chromHMM_state18_Quies")

# Tune up the tables and filter for significant p-values:
hyperfiltered <- hyper
hyperfiltered$logOddsRatio[which(hyperfiltered$qValue >= 0.05)] <- 1

hypofiltered <- hypo
hypofiltered$logOddsRatio[which(hypofiltered$qValue >= 0.05)] <- 1

# Generate the dataframe from the table:
hyperfiltered2 <- dcast(hyperfiltered, collection ~ cellType, value.var = "logOddsRatio")
hypofiltered2 <- dcast(hypofiltered, collection ~ cellType, value.var = "logOddsRatio")

rownames(hyperfiltered2) <- hyperfiltered2$collection
hyperfiltered2 <- hyperfiltered2[,-1]
rownames(hypofiltered2) <- hypofiltered2$collection
hypofiltered2 <- hypofiltered2[,-1]

hyperfiltered2 <- hyperfiltered2[orderstates,]
hypofiltered2 <- hypofiltered2[orderstates,]

hyperfiltered3 <- hyperfiltered2
hypofiltered3 <- hypofiltered2

hyperfiltered3$collection <- as.character(rownames(hyperfiltered3))
hypofiltered3$collection <- as.character(rownames(hypofiltered3))

hyperfiltered3 <- melt(hyperfiltered3)
hypofiltered3 <- melt(hypofiltered3)

hyperfiltered2[is.na(hyperfiltered2)] <- 1
hypofiltered2[is.na(hypofiltered2)] <- 1

hyperfiltered_encode <- hyperfiltered2[, c("Mobilized_CD34_Primary_Cells_Female", "CD19_Primary_Cells_Peripheral_UW", "GM12878_Lymphoblastoid" , "Dnd41_TCell_Leukemia", "K562_Leukemia")]

hypofiltered_encode <- hypofiltered2[, c("Mobilized_CD34_Primary_Cells_Female", "CD19_Primary_Cells_Peripheral_UW", "GM12878_Lymphoblastoid" , "Dnd41_TCell_Leukemia", "K562_Leukemia")]

# Generate the heatmaps from the table:
annotation_tissue2 <- data.frame(group = c("HSC", "LB", "LYMPHOBLAST", "LEUKEMIA", "LEUKEMIA"))
rownames(annotation_tissue2) <- colnames(hyperfiltered_encode)

pdf(file = paste(resultsDirectory, "Heatmaps_enrichments_chromatin_states_GERM_leukemia_short.pdf", sep = ""), paper = "a4r", height = 21, width = 28, onefile = T)

pheatmap(as.matrix(hyperfiltered_encode), scale = "none", color = colorRampPalette(c("#EFEFF0","#E8DB4B"))(100), border_color = "#FFFFFF", cluster_rows = F, cluster_cols = F, annotation_col = annotation_tissue2, clustering_method = "ward.D2", main = "Heatmap Enrichment OR chromatin states hyper GERM", fontsize_row = 5, cellwidth = 5, cellheight = 7, fontsize_col = 5)  
pheatmap(as.matrix(hypofiltered_encode), scale = "none", color = colorRampPalette(c("#EFEFF0","#E8DB4B"))(100), border_color = "#FFFFFF", cluster_rows = F, cluster_cols = F, annotation_col = annotation_tissue2, clustering_method = "ward.D2", main = "Heatmap Enrichment OR chromatin states hypo GERM", fontsize_row = 5, cellwidth = 5, cellheight = 7, fontsize_col = 5)  

dev.off()
