#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script to represent correlations between leukemias
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

# Script to represent correlation scatterplot between B cells and leukemias (Supplementary figure)

library(pheatmap)
library(RColorBrewer)
library(MASS)
library(RColorBrewer)
library(tidyr)
library(GenomicRanges)
library(annotatr)

rf <- colorRampPalette(rev(brewer.pal(11,'Spectral')))
r <- rf(32)

tablesDirectory <- ""
resultsDirectory <- ""

# Load Tables
bVals <- read.csv(paste(tablesDirectory, "bVals.txt", sep = "/"), sep = "\t", stringsAsFactors = F)
phenoData <- read.csv(paste(tablesDirectory, "phenoData2.csv", sep = "/"), sep = ",", stringsAsFactors = F)
rownames(phenoData) <- phenoData$Sample_Name

###############

annotationEPIC <- read.csv(paste(tablesDirectory, "annotation_EPIC.txt", sep = "/"), sep = "\t", stringsAsFactors = F)
annotationEPIC$pos <- annotationEPIC$pos + 600
annotationEPIC$end <-  annotationEPIC$end + 600
annotationEPIC$strand <- "*"
enhancersB <- read.csv(paste(tablesDirectory, "CD19+_EP.txt", sep = "/"), sep = "\t", stringsAsFactors = F)
enhancersB <- enhancersB[,c("chr", "start", "end", "strand", "ENSGID", "SYMBOL", "Chr_2", "Start_2", "Score")]
enhancersB <- enhancersB[which(enhancersB$Score > 2.1),]
rownames(enhancersB) <- c(1:nrow(enhancersB))

granges_annotationEPIC <- makeGRangesFromDataFrame(df = annotationEPIC, start.field = "pos", keep.extra.columns = T)
granges_enhancersB <- makeGRangesFromDataFrame(df = enhancersB, keep.extra.columns = T)

overlaps <- findOverlaps(query = granges_annotationEPIC, subject = granges_enhancersB)
overlaps <- as.data.frame(overlaps)
overlaps <- overlaps[!duplicated(overlaps$queryHits),]

annotationEPIC_filtered <- annotationEPIC[overlaps$queryHits,]
rownames(annotationEPIC_filtered) <- annotationEPIC_filtered$Name
bVals_filtered <- bVals[rownames(annotationEPIC_filtered),]
DMPs_Bcells <- read.csv(paste(tablesDirectory, "bVals_B_vs_ProB.txt", sep = "/"), sep = "\t", stringsAsFactors = F)
bVals_filtered <- bVals_filtered[which(rownames(bVals_filtered) %in% rownames(DMPs_Bcells)), which(phenoData$Group == "B_Lymphocytes_CD19" | phenoData$Group == "MLL-AF4" | phenoData$Group == "MLL-AF9" | phenoData$Group == "MLL GERMLINE" | phenoData$Group == "B-PROGENITORS")]

# Filter tables for matrix correlations
bVals_filtered_AF4 <- bVals_filtered[,which(phenoData$Group == "B_Lymphocytes_CD19" | phenoData$Group == "MLL-AF4")]
phenoData_filtered_AF4 <- phenoData[colnames(bVals_filtered_AF4),]
bVals_filtered_AF4$Mean_AF4 <- rowMeans(bVals_filtered_AF4[,which(phenoData_filtered_AF4$Group == "MLL-AF4")])

bVals_filtered_AF9 <- bVals_filtered[,which(phenoData$Group == "B_Lymphocytes_CD19" | phenoData$Group == "MLL-AF9")]
phenoData_filtered_AF9 <- phenoData[colnames(bVals_filtered_AF9),]
bVals_filtered_AF9$Mean_AF9 <- rowMeans(bVals_filtered_AF9[,which(phenoData_filtered_AF9$Group == "MLL-AF9")])

bVals_filtered_GERM <- bVals_filtered[,which(phenoData$Group == "B_Lymphocytes_CD19" | phenoData$Group == "MLL GERMLINE")]
phenoData_filtered_GERM <- phenoData[colnames(bVals_filtered_GERM),]
bVals_filtered_GERM$Mean_GERM <- rowMeans(bVals_filtered_GERM[,which(phenoData_filtered_GERM$Group == "MLL GERMLINE")])

# Plot figures
par(pty="s")
pdf(file= paste(resultsDirectory, "Correlation_leukemias_B_cells_enhancers.pdf", sep = "/"), height=8, width=8, onefile=TRUE)
AF4 <- kde2d(bVals_filtered_AF4$Mean_AF4, bVals_filtered_AF4$X8, lims = c(0,1,0,1), n = 500)
pearson_all <- cor(bVals_filtered_AF4$Mean_AF4, bVals_filtered_AF4$X8)
p.value_all <- cor.test(bVals_filtered_AF4$Mean_AF4, bVals_filtered_AF4$X8)$p.value
image(AF4, col=r, main = "Heat Scatterplot AF4 vs B cells", ylab = "B cells", xlab = "AF4")
abline(lm(Mean_AF4 ~ X8, data = bVals_filtered_AF4), col = "white", lwd = 3, lty = 2)
text(0.20, 0.80, paste("pearson corr = ", round(pearson_all, 2), sep = ""), col = "white")
text(0.20, 0.75, paste("p.value = ", round(p.value_all, 2), sep = ""), col = "white")

AF9 <- kde2d(bVals_filtered_AF9$Mean_AF9, bVals_filtered_AF9$X8, lims = c(0,1,0,1), n = 500)
pearson_all <- cor(bVals_filtered_AF9$Mean_AF9, bVals_filtered_AF9$X8)
p.value_all <- cor.test(bVals_filtered_AF9$Mean_AF9, bVals_filtered_AF9$X8)$p.value
image(AF9, col=r, main = "Heat Scatterplot AF9 vs B cells", ylab = "B cells", xlab = "AF9")
abline(lm(Mean_AF9 ~ X8, data = bVals_filtered_AF9), col = "white", lwd = 3, lty = 2)
text(0.20, 0.80, paste("pearson corr = ", round(pearson_all, 2), sep = ""), col = "white")
text(0.20, 0.75, paste("p.value = ", round(p.value_all, 2), sep = ""), col = "white")

GERM <- kde2d(bVals_filtered_GERM$Mean_GERM, bVals_filtered_GERM$X8, lims = c(0,1,0,1), n = 500)
pearson_all <- cor(bVals_filtered_GERM$Mean_GERM, bVals_filtered_GERM$X8)
p.value_all <- cor.test(bVals_filtered_GERM$Mean_GERM, bVals_filtered_GERM$X8)$p.value
image(GERM, col=r, main = "Heat Scatterplot GERM vs B cells", ylab = "B cells", xlab = "GERM")
abline(lm(Mean_GERM ~ X8, data = bVals_filtered_GERM), col = "white", lwd = 3, lty = 2)
text(0.20, 0.80, paste("pearson corr = ", round(pearson_all, 2), sep = ""), col = "white")
text(0.20, 0.75, paste("p.value = ", round(p.value_all, 2), sep = ""), col = "white")

annotation_control_case <- data.frame(group = as.character(phenoData$Group))
rownames(annotation_control_case) <- colnames(bVals_filtered)
pheatmap(bVals_filtered, cluster_rows = T, cluster_cols = T, color = colorRampPalette((brewer.pal(n = 7, name = "Blues")))(100), annotation = annotation_control_case, clustering_method = "ward.D2", main = "Heatmap Bcell enhancers")

dev.off()
dev.off()

# Annotate regions enhancers:
annotation_BenhancersEPIC <- annotationEPIC_filtered[rownames(bVals_filtered),]
rownames(annotation_BenhancersEPIC) <- c(1:nrow(annotation_BenhancersEPIC))
enhancersB <- read.csv(paste(tablesDirectory, "CD19+_EP.txt", sep = "/"), sep = "\t", stringsAsFactors = F)
enhancersB <- enhancersB[,c("chr", "start", "end", "strand", "ENSGID", "SYMBOL", "Chr_2", "Start_2", "Score")]
enhancersB <- enhancersB[which(enhancersB$Score > 2.1),]
rownames(enhancersB) <- c(1:nrow(enhancersB))
enhancersB$strand <- "*"

granges_enhancersB <- makeGRangesFromDataFrame(df = enhancersB, keep.extra.columns = T)
granges_BenhancersEPIC <- makeGRangesFromDataFrame(df = annotation_BenhancersEPIC, start.field = "pos", keep.extra.columns = T)
ranges <- subsetByOverlaps(granges_BenhancersEPIC,granges_enhancersB) 
hits <- findOverlaps(granges_BenhancersEPIC, granges_enhancersB)
ENSGID <- CharacterList(split(granges_enhancersB$ENSGID[subjectHits(hits)], queryHits(hits)))
SYMBOL <- CharacterList(split(granges_enhancersB$SYMBOL[subjectHits(hits)], queryHits(hits)))
mcols(ranges) <- DataFrame(mcols(ranges), ENSGID, SYMBOL)
annotationEPIC_BenhancersEPIC <- as.data.frame(ranges)

write.table(annotationEPIC_BenhancersEPIC, file = paste(tablesDirectory, "annotation_enhancers_Bcells_array.txt", sep = "/"), sep = "\t", col.names = T, row.names = F)

cor(bVals_filtered_AF4$Mean_AF4, bVals_filtered_AF4$X8)
cor(bVals_filtered_AF9$Mean_AF9, bVals_filtered_AF9$X8)
cor(bVals_filtered_GERM$Mean_GERM, bVals_filtered_GERM$X8)

enhancers <- unique(paste(enhancersB$chr, enhancersB$start, enhancersB$end, enhancersB$strand))


