#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script for the comparison of DNA methylation / Gene expression clusters
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

library(ggplot2)
library(gridExtra)
library(genefilter)
library(tidyr)
library(ggbio)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(grid)
library(lattice)
library(reshape2)
library(dendextend)

# Load the data:
phenoData <- read.csv("phenoData2.csv", header = T, sep = ",", stringsAsFactors = F)
rownames(phenoData) <- phenoData$Sample_Name
phenoData$COD_ANT[52:56] <- c("ProB_01", "ProB_02", "ProB_03", "ProB_04", "ProB_05")

bVals <- read.csv("bVals.txt", header = T, sep = "\t", stringsAsFactors = F)
bVals <- bVals[complete.cases(bVals),]
bVals <- bVals[,phenoData$Sample_Name]
colnames(bVals) <- phenoData$COD_ANT

RNAseq_FPKM <- read.table(file = "RSEM_RNAseq_genes_FPKM_matrix.tsv", header = T, sep = "\t")
rownames(RNAseq_FPKM) <- RNAseq_FPKM$ENSG_name
RNAseq_FPKM$MA4_01 <- as.numeric(as.character(RNAseq_FPKM$MA4_01))
RNAseq_FPKM <- RNAseq_FPKM[,-c(1,2)]
RNAseq_FPKM <- RNAseq_FPKM + 1
RNAseq_FPKM <- log2(RNAseq_FPKM)
RNAseq_FPKM <- RNAseq_FPKM[complete.cases(RNAseq_FPKM),]

# Get same samples RNAseq / Methylation arrays: 
phenoData_filtered <- phenoData[which(phenoData$RNA_expression == "YES"),]
colnames(bVals) <- phenoData$COD_ANT
bVals <- bVals[,phenoData_filtered$COD_ANT]
RNAseq_FPKM <- RNAseq_FPKM[, phenoData_filtered$COD_ANT]

# Compute distance matrix
res.dist_EXP <- dist(t(RNAseq_FPKM), method = "euclidean")
res.dist_Meth <- dist(t(bVals), method = "euclidean")

# Compute 2 hierarchical clusterings
hc1 <- hclust(res.dist_EXP, method = "ward.D2")
hc2 <- hclust(res.dist_Meth, method = "ward.D2")

# Create two dendrograms
dend1 <- as.dendrogram (hc1)
dend2 <- as.dendrogram (hc2)

# Create a list to hold dendrograms
dend_list <- dendlist(dend1, dend2)

# Align and plot two dendrograms side by side
pdf(file="tanglegram_exp_meth.pdf", height = 4, width = 5, onefile = TRUE)
dendlist(dend1, dend2) %>%
  untangle(method = "step1side") %>% 
  tanglegram(common_subtrees_color_branches = TRUE, highlight_distinct_edges = FALSE)
dev.off()

# Compute alignment quality. Lower value = good alignment quality
dendlist(dend1, dend2) %>%
  untangle(method = "step1side") %>% 
  entanglement(common_subtrees_color_branches = TRUE, highlight_distinct_edges = FALSE)

# Cophenetic correlation coefficient
cor_cophenetic(dend1, dend2)
