#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script for the analysis of RNAseq data with CEMITOOLS
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

resultsDirectory <- ""
tablesDirectory <- ""

library(CEMiTool)
library(tidyr)
library(ggplot2)

n_genes = 663

# read interactions file
int_df <- read.csv(paste(tablesDirectory, "interactions.tsv", sep = "/"), sep = "\t", stringsAsFactors = F)
int_df <- int_df[complete.cases(int_df),]
head(int_df)

# read GMT file
gmt_c2 <- read_gmt(paste(tablesDirectory, "GeneSets/c2.all.v5.2.symbols.gmt", sep = "/"))
gmt_c3 <- read_gmt(paste(tablesDirectory, "GeneSets/c3.all.v5.2.symbols.gmt", sep = "/"))
gmt_c5 <- read_gmt(paste(tablesDirectory, "GeneSets/c5.all.v5.2.symbols.gmt", sep = "/"))
gmt_c6 <- read_gmt(paste(tablesDirectory, "GeneSets/c6.all.v5.2.symbols.gmt", sep = "/"))
gmt_h <- read_gmt(paste(tablesDirectory, "GeneSets/h.all.v5.2.symbols.gmt", sep = "/"))

# Load and tune up expression data:
phenoData <- read.csv(paste(tablesDirectory, "phenoData.csv", sep = "/"), sep = "\t", stringsAsFactors = F)
#phenoData <- phenoData[which(phenoData$Sample_Group != "Bcell"),]
expr0 <- read.csv(paste(tablesDirectory, "RSEM_RNAseq_genes_FPKM_matrix.tsv", sep = "/"), sep = "\t", stringsAsFactors = F)
expr0 <- separate(data = expr0, col = gene_id, into = c("ensembl_id", "gene_symbol"), sep = "_", remove = T)
expr0 <- expr0[!duplicated(expr0$gene_symbol), ]
rownames(expr0) <- expr0$gene_symbol
expr0 <- expr0[,phenoData$Sample_Name_RNAseq]
expr0 <- log(expr0, 2)
expr0 <- expr0[complete.cases(expr0),]
expr0  <- expr0 [is.finite(rowSums(expr0)),]
head(expr0[,1:4])

# load your sample annotation data
sample_annot <- data.frame(SampleName = phenoData$Sample_Name_RNAseq, Class = phenoData$Sample_Group)
head(sample_annot)

cem <- cemitool(expr0,sample_annot, filter = TRUE, n_genes = n_genes, ora_pval = 0.001, verbose = TRUE, diss_thresh = 0.8, filter_pval = 0.05, min_ngen = 30)
sample_annotation(cem, sample_name_column="SampleName", class_column="Class") <- sample_annot
cem

# Check the number of modules:
nmodules(cem)
head(module_genes(cem))

pdf(paste(n_genes, "CEMiTools_leukemia.pdf", sep = "_"), paper="a4r", height=12, width=16, onefile=TRUE)
# plot gene expression within each module
cem <- plot_profile(cem)
plots <- show_plot(cem, "profile")
plots

# perform over representation analysis
cem <- mod_ora(cem, gmt_c2)
cem <- plot_ora(cem)
plots <- show_plot(cem, "ora")
plots

cem <- mod_ora(cem, gmt_c3)
cem <- plot_ora(cem)
plots <- show_plot(cem, "ora")
plots

cem <- mod_ora(cem, gmt_c5)
cem <- plot_ora(cem)
plots <- show_plot(cem, "ora")
plots

cem <- mod_ora(cem, gmt_c6)
cem <- plot_ora(cem)
plots <- show_plot(cem, "ora")
plots

cem <- mod_ora(cem, gmt_h)
cem <- plot_ora(cem)
plots <- show_plot(cem, "ora")
plots

# plot interactions
interactions_data(cem) <- int_df
cem <- plot_interactions(cem)
plots <- show_plot(cem, "interaction")
plots

dev.off()

# Generate report:
generate_report(cem, output_format=c("pdf_document", "html_document"))
write_files(cem, directory="./CEMiTools_leukemia", force=TRUE)






