#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script for the calculation of ELMER correlations
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

options("width"=110) #  using R in terminal mode on Linux, making easier the inspection of data sets

###################################################################
#####                       READING DATA                      #####
###################################################################

# Set up the directory for the results:
resultsDirectory <- ""
tablesDirectory <- ""

# Load the required packages:
#devtools::install_github(repo = "lijingya/ELMER")
library(ELMER)
library(ELMER.data)
library(RColorBrewer)
library(ggplot2)
library(tidyr)
library(GenomicRanges)

data("EPIC.manifest")
EPIC.manifest <- as.data.frame(EPIC.manifest)

# Load the data:
phenoData <- read.csv(paste(tablesDirectory, "phenoData2.csv", sep = "/"), header = T, sep = ",", stringsAsFactors = F)
rownames(phenoData) <- phenoData$Sample_Name
phenoData_AF4_meth_RNA <- phenoData[which(phenoData$RNA_expression == "YES" & (phenoData$Group == "B-PROGENITORS" | phenoData$Group == "MLL-AF4")),]
phenoData_AF4_meth_RNA$Group <- factor(phenoData_AF4_meth_RNA$Group, levels = c("MLL-AF4", "B-PROGENITORS"))
phenoData_AF9_meth_RNA <- phenoData[which(phenoData$RNA_expression == "YES" & (phenoData$Group == "B-PROGENITORS" | phenoData$Group == "MLL-AF9")),]
phenoData_AF9_meth_RNA$Group <- factor(phenoData_AF9_meth_RNA$Group, levels = c("MLL-AF9", "B-PROGENITORS"))
phenoData_GERM_meth_RNA <- phenoData[which(phenoData$RNA_expression == "YES" & (phenoData$Group == "B-PROGENITORS" | phenoData$Group == "MLL GERMLINE")),]
phenoData_GERM_meth_RNA$Group <- factor(phenoData_GERM_meth_RNA$Group, levels = c("MLL GERMLINE", "B-PROGENITORS"))
phenoData_GERM_meth_RNA <- phenoData_GERM_meth_RNA[order(phenoData_GERM_meth_RNA$Group, phenoData_GERM_meth_RNA$Sample_Name),]
phenoData_COMMON_meth_RNA <- phenoData[which(phenoData$RNA_expression == "YES" & (phenoData$Group == "B-PROGENITORS" | phenoData$Group == "MLL-AF4" | phenoData$Group == "MLL-AF9" | phenoData$Group == "MLL GERMLINE")),]
phenoData_COMMON_meth_RNA$tissue <- factor(phenoData_COMMON_meth_RNA$tissue, levels = c("Blood", "fetal_liver"))
phenoData_COMMON_meth_RNA <- phenoData_COMMON_meth_RNA[order(phenoData_COMMON_meth_RNA$tissue, phenoData_COMMON_meth_RNA$Sample_Name),]

bVals <- read.csv(paste(tablesDirectory, "bVals.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
bVals <- bVals[which(rownames(bVals) %in% rownames(EPIC.manifest)),]
bVals <- bVals[complete.cases(bVals),]

DMPs_AF4_hyper <- read.csv(paste(tablesDirectory, "annotation_AF4_hyper_probes.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
bVals_AF4_hyper <- bVals[which(rownames(bVals) %in% DMPs_AF4_hyper$Name), phenoData_AF4_meth_RNA$Sample_Name]
bVals_AF4_hyper <- bVals_AF4_hyper[complete.cases(bVals_AF4_hyper),]
bVals_AF4_hyper <- as.matrix(bVals_AF4_hyper)

DMPs_AF4_hypo <- read.csv(paste(tablesDirectory, "annotation_AF4_hypo_probes.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
bVals_AF4_hypo <- bVals[which(rownames(bVals) %in% DMPs_AF4_hypo$Name), phenoData_AF4_meth_RNA$Sample_Name]
bVals_AF4_hypo <- bVals_AF4_hypo[complete.cases(bVals_AF4_hypo),]
bVals_AF4_hypo <- as.matrix(bVals_AF4_hypo)

DMPs_AF9_hyper <- read.csv(paste(tablesDirectory, "annotation_AF9_hyper_probes.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
bVals_AF9_hyper <- bVals[which(rownames(bVals) %in% DMPs_AF9_hyper$Name), phenoData_AF9_meth_RNA$Sample_Name]
bVals_AF9_hyper <- bVals_AF9_hyper[complete.cases(bVals_AF9_hyper),]
bVals_AF9_hyper <- as.matrix(bVals_AF9_hyper)

DMPs_AF9_hypo <- read.csv(paste(tablesDirectory, "annotation_AF9_hypo_probes.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
bVals_AF9_hypo <- bVals[which(rownames(bVals) %in% DMPs_AF9_hypo$Name), phenoData_AF9_meth_RNA$Sample_Name]
bVals_AF9_hypo <- bVals_AF9_hypo[complete.cases(bVals_AF9_hypo),]
bVals_AF9_hypo <- as.matrix(bVals_AF9_hypo)

DMPs_GERM_hyper <- read.csv(paste(tablesDirectory, "annotation_GERM_hyper_probes.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
bVals_GERM_hyper <- bVals[which(rownames(bVals) %in% DMPs_GERM_hyper$Name), phenoData_GERM_meth_RNA$Sample_Name]
bVals_GERM_hyper <- bVals_GERM_hyper[complete.cases(bVals_GERM_hyper),]
bVals_GERM_hyper <- as.matrix(bVals_GERM_hyper)

DMPs_GERM_hypo <- read.csv(paste(tablesDirectory, "annotation_GERM_hypo_probes.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
bVals_GERM_hypo <- bVals[which(rownames(bVals) %in% DMPs_GERM_hypo$Name), phenoData_GERM_meth_RNA$Sample_Name]
bVals_GERM_hypo <- bVals_GERM_hypo[complete.cases(bVals_GERM_hypo),]
bVals_GERM_hypo <- as.matrix(bVals_GERM_hypo)

DMPs_COMMON_hyper <- read.csv(paste(tablesDirectory, "annotation_COMMON_hyper_probes.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
bVals_COMMON_hyper <- bVals[which(rownames(bVals) %in% DMPs_COMMON_hyper$Name), phenoData_COMMON_meth_RNA$Sample_Name]
bVals_COMMON_hyper <- bVals_COMMON_hyper[complete.cases(bVals_COMMON_hyper),]
bVals_COMMON_hyper <- as.matrix(bVals_COMMON_hyper)

DMPs_COMMON_hypo <- read.csv(paste(tablesDirectory, "annotation_COMMON_hypo_probes.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
bVals_COMMON_hypo <- bVals[which(rownames(bVals) %in% DMPs_COMMON_hypo$Name), phenoData_COMMON_meth_RNA$Sample_Name]
bVals_COMMON_hypo <- bVals_COMMON_hypo[complete.cases(bVals_COMMON_hypo),]
bVals_COMMON_hypo <- as.matrix(bVals_COMMON_hypo)

RNAseq <- read.table(file = paste(tablesDirectory, "RSEM_RNAseq_genes_FPKM_matrix.tsv", sep = "/"), header = T, sep = "\t")
rownames(RNAseq) <- RNAseq$ENSG_name

RNAseq_AF4 <- RNAseq[, phenoData_AF4_meth_RNA$COD_ANT]
RNAseq_AF4 <- log(RNAseq_AF4,2)
RNAseq_AF4 <- RNAseq_AF4[complete.cases(RNAseq_AF4),]
RNAseq_AF4 <- RNAseq_AF4[is.finite(rowSums(RNAseq_AF4)),]
colnames(RNAseq_AF4) <- phenoData_AF4_meth_RNA$Sample_Name
phenoData_AF4_meth_RNA$samples <- phenoData_AF4_meth_RNA$Sample_Name
phenoData_AF4_meth_RNA$group <- phenoData_AF4_meth_RNA$Group
phenoData_AF4_meth_RNA <- phenoData_AF4_meth_RNA[, c("samples", "group")]

RNAseq_AF9 <- RNAseq[, phenoData_AF9_meth_RNA$COD_ANT]
RNAseq_AF9 <- log(RNAseq_AF9,2)
RNAseq_AF9 <- RNAseq_AF9[complete.cases(RNAseq_AF9),]
RNAseq_AF9 <- RNAseq_AF9[is.finite(rowSums(RNAseq_AF9)),]
colnames(RNAseq_AF9) <- phenoData_AF9_meth_RNA$Sample_Name
phenoData_AF9_meth_RNA$samples <- phenoData_AF9_meth_RNA$Sample_Name
phenoData_AF9_meth_RNA$group <- phenoData_AF9_meth_RNA$Group
phenoData_AF9_meth_RNA <- phenoData_AF9_meth_RNA[, c("samples", "group")]

RNAseq_GERM <- RNAseq[, phenoData_GERM_meth_RNA$COD_ANT]
RNAseq_GERM <- log(RNAseq_AF4,2)
RNAseq_GERM <- RNAseq_GERM[complete.cases(RNAseq_GERM),]
RNAseq_GERM <- RNAseq_GERM[is.finite(rowSums(RNAseq_GERM)),]
colnames(RNAseq_GERM) <- phenoData_GERM_meth_RNA$Sample_Name
phenoData_GERM_meth_RNA$samples <- phenoData_GERM_meth_RNA$Sample_Name
phenoData_GERM_meth_RNA$group <- phenoData_GERM_meth_RNA$Group
phenoData_GERM_meth_RNA <- phenoData_GERM_meth_RNA[, c("samples", "group")]

RNAseq_COMMON <- RNAseq[, phenoData_COMMON_meth_RNA$COD_ANT]
RNAseq_COMMON <- log(RNAseq_COMMON,2)
RNAseq_COMMON <- RNAseq_COMMON[complete.cases(RNAseq_COMMON),]
RNAseq_COMMON <- RNAseq_COMMON[is.finite(rowSums(RNAseq_COMMON)),]
colnames(RNAseq_COMMON) <- phenoData_COMMON_meth_RNA$Sample_Name
phenoData_COMMON_meth_RNA$samples <- phenoData_COMMON_meth_RNA$Sample_Name
phenoData_COMMON_meth_RNA$group <- phenoData_COMMON_meth_RNA$tissue
phenoData_COMMON_meth_RNA <- phenoData_COMMON_meth_RNA[, c("samples", "group")]

elmer_mae_AF4_hyper <- createMAE(exp = RNAseq_AF4, met =  bVals_AF4_hyper, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_AF4_meth_RNA, linearize.exp = TRUE, save.filename = "mae_AF4_hyper_filtered.rda")
elmer_mae_AF4_hypo <- createMAE(exp = RNAseq_AF4, met =  bVals_AF4_hypo, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_AF4_meth_RNA, linearize.exp = TRUE, save.filename = "mae_AF4_hypo_filtered.rda")

elmer_mae_AF9_hyper <- createMAE(exp = RNAseq_AF9, met =  bVals_AF9_hyper, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_AF9_meth_RNA, linearize.exp = TRUE, save.filename = "mae_AF9_hyper_filtered.rda")
elmer_mae_AF9_hypo <- createMAE(exp = RNAseq_AF9, met =  bVals_AF9_hypo, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_AF9_meth_RNA, linearize.exp = TRUE, save.filename = "mae_AF9_hypo_filtered.rda")

elmer_mae_GERM_hyper <- createMAE(exp = RNAseq_GERM, met =  bVals_GERM_hyper, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_GERM_meth_RNA, linearize.exp = TRUE, save.filename = "mae_GERM_hyper_filtered.rda")
elmer_mae_GERM_hypo <- createMAE(exp = RNAseq_GERM, met =  bVals_GERM_hypo, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_GERM_meth_RNA, linearize.exp = TRUE, save.filename = "mae_GERM_hypo_filtered.rda")

elmer_mae_COMMON_hyper <- createMAE(exp = RNAseq_COMMON, met =  bVals_COMMON_hyper, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_COMMON_meth_RNA, linearize.exp = TRUE, save.filename = "mae_COMMON_hyper_filtered.rda")
elmer_mae_COMMON_hypo <- createMAE(exp = RNAseq_COMMON, met =  bVals_COMMON_hypo, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_COMMON_meth_RNA, linearize.exp = TRUE, save.filename = "mae_COMMON_hypo_filtered.rda")

bVals_AF4 <- rbind(bVals_AF4_hyper, bVals_AF4_hypo)
bVals_AF9 <- rbind(bVals_AF9_hyper, bVals_AF9_hypo)
bVals_GERM <- rbind(bVals_GERM_hyper, bVals_GERM_hypo)
bVals_COMMON <- rbind(bVals_COMMON_hyper, bVals_COMMON_hypo)

elmer_mae_AF4 <- createMAE(exp = RNAseq_AF4, met =  bVals_AF4, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_AF4_meth_RNA, linearize.exp = TRUE, save.filename = "mae_AF4_filtered.rda")
elmer_mae_AF9 <- createMAE(exp = RNAseq_AF9, met =  bVals_AF9, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_AF9_meth_RNA, linearize.exp = TRUE, save.filename = "mae_AF9_filtered.rda")
elmer_mae_GERM <- createMAE(exp = RNAseq_GERM, met =  bVals_GERM, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_GERM_meth_RNA, linearize.exp = TRUE, save.filename = "mae_GERM_filtered.rda")
elmer_mae_COMMON <- createMAE(exp = RNAseq_COMMON, met =  bVals_COMMON, TCGA = FALSE, save = TRUE, met.platform = "EPIC", genome = "hg19", colData = phenoData_COMMON_meth_RNA, linearize.exp = TRUE, save.filename = "mae_COMMON_filtered.rda")

