#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script for the calculation of ELMER correlations
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

options("width"=110) #  using R in terminal mode on Linux, making easier the inspection of data sets

###################################################################
#####                       READING DATA                      #####
###################################################################

# Set up the directory for the results:
resultsDirectory <- ""
tablesDirectory <- ""

# Load the required packages:
library(SummarizedExperiment)
library(ELMER)
library(ELMER.data)
library(RColorBrewer)
library(tidyr)
library(GenomicRanges)
library(plyr)
library(doParallel)
library(magrittr)


# Load the data:
mae_hyper <- get(load("./mae_AF4_hyper_filtered.rda"))
mae_hypo <- get(load("./mae_AF4_hypo_filtered.rda"))
load("./Homo_sapiens_genes__GRCh37_p13_.rda")

# Get near genes:
nearGenes_hyper <- GetNearGenes(data = mae_hyper, probes = rownames(mae_hyper@ExperimentList$`DNA methylation`), numFlankingGenes = 5, cores = 3)
nearGenes_hypo <- GetNearGenes(data = mae_hypo, probes = rownames(mae_hypo@ExperimentList$`DNA methylation`), numFlankingGenes = 5, cores = 3)

Hyper.pair <- get.pair(data = mae_hyper, group.col = "group", group1 = "MLL-AF4", group2 = "B-PROGENITORS", nearGenes = nearGenes_hyper, minSubgroupFrac = 1.0, permu.size = 2000, raw.pvalue = 0.05, Pe = 0.001, mode = "supervised", diff.dir = "hyper", cores = 4, diffExp = TRUE, save = FALSE)
write.table(Hyper.pair, file = paste(tablesDirectory,"hyper_pairs_AF4_new.txt", sep = "/"), sep = "\t", row.names = T, col.names = T, quote = T)
Hypo.pair <- get.pair(data = mae_hypo, group.col = "group", group1 = "MLL-AF4", group2 = "B-PROGENITORS", nearGenes = nearGenes_hypo, minSubgroupFrac = 1.0, permu.size = 2000, raw.pvalue = 0.05, Pe = 0.001, mode = "supervised", diff.dir = "hypo", cores = 4, diffExp = TRUE, save = FALSE)
write.table(Hypo.pair, file = paste(tablesDirectory,"hypo_pairs_AF4_new.txt", sep = "/"), sep = "\t", row.names = T, col.names = T, quote = T)

# Get enriched motifs:
data("EPIC.manifest")
EPIC.manifest <- as.data.frame(EPIC.manifest)

pdf(file = paste(resultsDirectory, "Risktable_AF4_hyper_motifs_new_2.pdf", sep = ""), paper = "a4r", height = 21, width = 28, onefile = T)
enriched.motif.hyper <- get.enriched.motif(data = mae_hyper, probes = Hyper.pair$Probe, min.incidence = 10, lower.OR = 1.1, background.probes = rownames(EPIC.manifest), min.motif.quality = c("BS"), save = FALSE, label = "AF4_hyper_new")
enriched.hyper <- read.csv("./getMotif.motif.enrichment_AF4_hyper_new.csv", sep = "\t")
motif.enrichment.plot(motif.enrichment = enriched.hyper, significant = list(OR = 1.5, lowerOR = 1.3), save = FALSE, summary = F)
dev.off()

pdf(file = paste(resultsDirectory, "Risktable_AF4_hypo_motifs_new_2.pdf", sep = ""), paper = "a4r", height = 21, width = 28, onefile = T)
enriched.motif.hypo <- get.enriched.motif(data = mae_hypo, probes = Hypo.pair$Probe, min.incidence = 10, lower.OR = 1.1, background.probes = rownames(EPIC.manifest), min.motif.quality = c("BS"), save = FALSE, label = "AF4_hypo_new")
enriched.hypo <- read.csv("./getMotif.motif.enrichment_AF4_hypo_new.csv", sep = "\t")
motif.enrichment.plot(motif.enrichment = enriched.hypo, significant = list(OR = 1.5, lowerOR = 1.3), save = FALSE, summary = F)
dev.off()

# Get TFs:
# transcription_factors <- getTF()
# write.table(transcription_factors, file = "./Tables/annotations/transcription_factor_list.txt", sep = "\t", row.names = F, quote = F)
transcription_factors <- read.csv(paste(tablesDirectory, "transcription_factor_list.txt", sep = "/"), sep = "\t", stringsAsFactors = F)
TF_hyper <- get.TFs(data = mae_hyper, TFs = transcription_factors, group.col = "group", group1 = "MLL-AF4", group2 = "B-PROGENITORS", minSubgroupFrac = 1.0, mode = "supervised", diff.dir = "hyper", enriched.motif = enriched.motif.hyper, save = FALSE, cores = 4, label = "hyper_AF4_new")
write.table(TF_hyper, file = "./Tables/TF_AF4_hyper_new.txt", sep = "\t", quote = F, row.names = T, col.names = T)
TF_hypo <- get.TFs(data = mae_hypo, TFs = transcription_factors, group.col = "group", group1 = "MLL-AF4", group2 = "B-PROGENITORS", minSubgroupFrac = 1.0, mode = "supervised", diff.dir = "hypo", enriched.motif = enriched.motif.hypo, save = FALSE, cores = 4, label = "hypo_AF4_new")
write.table(TF_hypo, file = "./Tables/TF_AF4_hypo_new.txt", sep = "\t", quote = F, row.names = T, col.names = T)
