#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script for the generation of methylation / expression plots
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

#################################################################################
###                 Generate plots expression / methylation                   ###
#################################################################################

library(ggplot2)
library(gridExtra)
library(genefilter)
library(tidyr)
library(ggbio)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(grid)
library(lattice)
library(reshape2)

# Load the data:
phenoData <- read.csv("phenoData2.csv", header = T, sep = ",", stringsAsFactors = F)
rownames(phenoData) <- phenoData$Sample_Name
phenoData$COD_ANT[52:56] <- c("ProB_01", "ProB_02", "ProB_03", "ProB_04", "ProB_05")

bVals <- read.csv("bVals.txt", header = T, sep = "\t", stringsAsFactors = F)
bVals <- bVals[complete.cases(bVals),]
bVals <- bVals[,phenoData$COD_ANT]

RNAseq_FPKM <- read.table(file = "RSEM_RNAseq_genes_FPKM_matrix.tsv", header = T, sep = "\t")
rownames(RNAseq_FPKM) <- RNAseq_FPKM$ENSG_name
RNAseq_FPKM$MA4_01 <- as.numeric(as.character(RNAseq_FPKM$MA4_01))
RNAseq_FPKM <- RNAseq_FPKM[,-c(1,2)]
RNAseq_FPKM <- RNAseq_FPKM + 1
RNAseq_FPKM <- log2(RNAseq_FPKM)
RNAseq_FPKM <- RNAseq_FPKM[complete.cases(RNAseq_FPKM),]

phenoRNA <- data.frame(Sample_Name = colnames(RNAseq_FPKM), Sample_Group = c(rep("MLL_AF4", 27), rep("MLL_AF9", 5), rep("nonMLL", 8), rep("BCP", 5), rep("Bcell", 3)))
phenoRNA$Sample_Group <- factor(phenoRNA$Sample_Group, levels = c("BCP", "MLL_AF4", "MLL_AF9", "nonMLL", "Bcell"))
                                
# Calculate GE for a given gene:
geneplot_FPKM <- function(genename, alias) {
  rnaseqmatrix <- data.frame(values = as.numeric(RNAseq_FPKM[which(rownames(RNAseq_FPKM) == genename),]), group = phenoRNA$Sample_Group)
  rnaseqmatrix$genename <- alias
  yp <- subset(rnaseqmatrix, values>0)
  sts <- boxplot.stats(yp$values)$stats
  #ggplot(rnaseqmatrix, aes(x=group, y=values, fill=group)) + geom_boxplot() + scale_fill_manual(values=c("#3288BD", "#D53E4F", "#2eaa6c", "#a59c31", "#763d91")) + coord_cartesian(ylim = c(sts[2]/2,max(sts)*2)) + labs(x = "Subtype", y = "Gene Expression (FPKM)") + theme(aspect.ratio=1, legend.position="none") + theme_bw() + ggtitle(label = alias)
  ggplot(rnaseqmatrix, aes(x=group, y=values, fill=group)) + geom_boxplot() + scale_fill_manual(values=c("#3288BD", "#D53E4F", "#2eaa6c", "#a59c31", "#763d91")) + labs(x = "Subtype", y = "Gene Expression (Log2 FPKM)") + theme(aspect.ratio=1, legend.position="none") + theme_bw() + ggtitle(label = alias)
}

pdf(file="explot.pdf", height = 4, width = 5, onefile = TRUE)
geneplot_FPKM("ENSG00000133740", "E2F5")
geneplot_FPKM("ENSG00000143507", "DUSP10")
geneplot_FPKM("ENSG00000175592", "FOSL1")
geneplot_FPKM("ENSG00000075426", "FOSL2")
geneplot_FPKM("ENSG00000170345", "FOS")
geneplot_FPKM("ENSG00000125740", "FOSB")
geneplot_FPKM("ENSG00000177606", "JUN")
geneplot_FPKM("ENSG00000171223", "JUNB")
geneplot_FPKM("ENSG00000130522", "JUND")
geneplot_FPKM("ENSG00000026508", "CD44")
geneplot_FPKM("ENSG00000159216", "RUNX1")
dev.off()


########### Get the whole matrix with 5mC data ###############
txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
annotationEPIC <- read.csv(file = "annotation_EPIC.txt", sep = "\t", stringsAsFactors = F) 
annotationEPIC <- separate(data = annotationEPIC, col = "GencodeCompV12_NAME", into = c("genesymbol", "other_gene_symbol"), sep = ";", remove = T)
annotationEPIC$chr2 <- gsub(pattern = "chr", replacement = "", x = annotationEPIC$chr)
rownames(annotationEPIC) <- annotationEPIC$Name
annotationEPIC$strand <- "+"
annotationEPIC <- annotationEPIC[rownames(bVals),]
annotationEPIC <- annotationEPIC[order(annotationEPIC$chr2, annotationEPIC$pos),]
colnames(annotationEPIC)[2] <- "start"
bVals_media <- bVals[rownames(annotationEPIC),]
bVals_media$Average_Bpro <- rowMeans(bVals_media[,c(39, 52:56)])
bVals_media$Average_AF4 <- rowMeans(bVals_media[,c(1:37)])
bVals_media$Average_AF9 <- rowMeans(bVals_media[,c(40:51)])
bVals_media$Average_GERM <- rowMeans(bVals_media[,c(57:76)])
bVals_media$Average_Bcell <-bVals_media[,c(38)]
bVals_media <- bVals_media[, c("Average_Bpro", "Average_AF4", "Average_AF9", "Average_GERM", "Average_Bcell")]

# Represent Meth plot for a given gene:
 methylationplot <- function(geneofinterest, chromosome, start, end){
   annotationEPIC_filtered <- annotationEPIC[which(annotationEPIC$chr == chromosome & annotationEPIC$start >= start & annotationEPIC$end <= end),]
   bVals_media <- bVals_media[rownames(annotationEPIC_filtered),]
   bVals_media <- bVals_media[rownames(annotationEPIC_filtered),]
   annotationEPIC_filtered2 <- annotationEPIC_filtered
   rownames(annotationEPIC_filtered2) <- c(1:length(annotationEPIC_filtered2[,1]))
   cpgs.new_5mC <- makeGRangesFromDataFrame(annotationEPIC_filtered2, keep.extra.columns = F)
   values(cpgs.new_5mC)$Bpro <- bVals_media$Average_Bpro
   values(cpgs.new_5mC)$AF4 <- bVals_media$Average_AF4
  values(cpgs.new_5mC)$AF9 <- bVals_media$Average_AF9
   values(cpgs.new_5mC)$GERM <- bVals_media$Average_GERM
   values(cpgs.new_5mC)$Bcell <- bVals_media$Average_Bcell
   values(cpgs.new_5mC)$Name <- annotationEPIC_filtered2$Name
   values(cpgs.new_5mC)$Significant <- FALSE
   gene <- GRanges(chromosome, IRanges(start, end))
   genecoords <- autoplot(txdb, gene, fill = "brown", color = "brown")
   plotRangesLinkedToData(cpgs.new_5mC, stat.y = c(1:5), size = 1, sig = "Significant", sig.col = c("gray70", "red"), theme.stat = theme_bw(), annotation = genecoords, height = c(200, 1, 1, 1), plot = T)
 }
 
 pdf(file="methlot.pdf", height = 4, width = 5, onefile = TRUE)
 methylationplot("RUNX1", "chr21", 36160411, 36259925)		
 methylationplot("JUN", "chr1",	59244497,	59250781)	
 methylationplot("JUNB", "chr19",	12901231,	12905057)	
 methylationplot("JUND", "chr19",	18389430,	18392807)	
 methylationplot("FOS", "chr14",	75743362,	75750562)	
 methylationplot("FOSB", "chr19",	45967101,	45980609)	
 methylationplot("FOSL1", "chr11",	65659246,	65671092)	
 methylationplot("FOSL2", "chr2",	28612556,	28643852)	
 methylationplot("RUNX1", "chr21",	36127036,	36495739)		
 methylationplot("E2F5", "chr8",	86081973,	86127913)	
 dev.off()

# Get same samples RNAseq / Methylation arrays: 
phenoData_filtered <- phenoData[which(phenoData$RNA_expression == "YES"),]
colnames(bVals) <- phenoData$COD_ANT
bVals <- bVals[,phenoData_filtered$COD_ANT]
RNAseq_FPKM <- RNAseq_FPKM[, phenoData_filtered$COD_ANT]

# Calculate Corr Meth-GE for a given gene:
methexpplot <- function(CpG, genename, alias) {
  correlation <- cor(as.numeric(as.vector(bVals[which(rownames(bVals) == CpG),])), as.numeric(as.vector(RNAseq_FPKM[which(rownames(RNAseq_FPKM) == genename),])), method = "pearson")
  correlationmatrix <- data.frame(valuesx = as.numeric(bVals[which(rownames(bVals) == CpG),]), valuesy = as.numeric(RNAseq_FPKM[which(rownames(RNAseq_FPKM) == genename),]), group = c(phenoData_filtered$Group), color = c(phenoData_filtered$color))
  ggplot(correlationmatrix, aes(x=valuesx, y=valuesy)) + geom_point(size = 3, colour = correlationmatrix$color, alpha = .5) + geom_smooth(method = 'lm',size = 1, colour = 'black', se = F) + coord_cartesian(xlim = c(0, 1)) + annotate("text", label = paste("Cor = ", round(correlation,2)), x = Inf, y = Inf, size = 6, colour = "black", vjust = 2, hjust = 1.1) + labs(x = "B-Value", y = "Gene Expression (Log2 RSEM FPKM)") + theme(aspect.ratio=1) + theme_bw() + ggtitle(label = paste(CpG, "-", alias))
}	

par(pty="s")
pdf(file="methexplot_BRIP1_LMO2.pdf", height = 4, width = 4, onefile = TRUE)
methexpplot("cg06365535",	"ENSG00000136492", "BRIP1")
methexpplot("cg07893009",	"ENSG00000135363", "LMO2")
dev.off()


# Load the data:
phenoData <- read.csv("phenoData2.csv", header = T, sep = ",", stringsAsFactors = F)
rownames(phenoData) <- phenoData$Sample_Name
phenoData$COD_ANT[52:56] <- c("ProB_01", "ProB_02", "ProB_03", "ProB_04", "ProB_05")
phenoData$Group <- factor(phenoData$Group, levels = c("B-PROGENITORS", "MLL-AF4", "MLL-AF9", "MLL GERMLINE", "B_Lymphocytes_CD19"))

bVals <- read.csv("bVals.txt", header = T, sep = "\t", stringsAsFactors = F)
bVals <- bVals[complete.cases(bVals),]
bVals <- bVals[,phenoData$Sample_Name]

COMMON <- read.csv("filtered_pairs_COMMON_new_2.txt", sep = "\t", stringsAsFactors = F)
AF4 <- read.csv("filtered_pairs_AF4_new_2.txt", sep = "\t", stringsAsFactors = F)
AF9 <- read.csv("filtered_pairs_AF9_new_2.txt", sep = "\t", stringsAsFactors = F)
GERM <- read.csv("filtered_pairs_GERM_new_2.txt", sep = "\t", stringsAsFactors = F)

# Get probes for violin plots:
average_probeplot_bVals <- function(table, genename, alias) {
  filteredtable <- table[which(table$GeneID == genename),]
  bValsmatrix <- bVals[which(rownames(bVals) %in% filteredtable$Probe),]
  bValsmatrix_long <- melt(bValsmatrix)
  bValsmatrix_long$Group <- phenoData$Group[match(bValsmatrix_long$variable, phenoData$Sample_Name)]
  bValsmatrix$genename <- alias
  ggplot(bValsmatrix_long, aes(x=Group, y=value, fill=Group)) + geom_boxplot() + scale_fill_manual(values=c("#3288BD", "#D53E4F", "#2eaa6c", "#a59c31", "#763d91")) + labs(x = "Subtype", y = "Average meth correlated Probes (Log2 FPKM)") + theme(aspect.ratio=1, legend.position="none") + theme_bw() + ggtitle(label = alias)
}

par(pty="s")
pdf(file="methlot_FOSL_RUNX_E2F.pdf", height = 4, width = 5, onefile = TRUE)
genename <- "ENSG00000075426"
average_probeplot_bVals(COMMON, genename, "FOSL2")

genename <- "ENSG00000133740"
average_probeplot_bVals(COMMON, genename, "E2F5")

genename <- "ENSG00000159216"
average_probeplot_bVals(COMMON, genename, "RUNX1")

genename <- "ENSG00000026508"
average_probeplot_bVals(COMMON, genename, "CD44")

dev.off()

