#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script for network representation and TFBS visualization
###          
###
### Authors: Juan Ramon Tejedor (Dec 2020)
#############################################################################

#################################################################################
###               Represent average methylation TF expression                 ###
#################################################################################

options("width"=110) #  using R in terminal mode on Linux, making easier the inspection of data sets

###################################################################
#####                       READING DATA                      #####
###################################################################

# Set up the directory for the results:
resultsDirectory <- ""
tablesDirectory <- ""

# Load the required packages:
#devtools::install_github(repo = "lijingya/ELMER")
library(ELMER)
library(ELMER.data)
library(RColorBrewer)
library(ggplot2)
library(tidyr)
library(GenomicRanges)
library(edgeR)
library(magrittr)
library(plyr)
library(GenomicInteractions)
library(extrafont)
#font_import()
loadfonts()

data("EPIC.manifest")
EPIC.manifest <- as.data.frame(EPIC.manifest)

# Load the data:
phenoData <- read.csv("./Tables/phenoData2.csv", header = T, sep = ",", stringsAsFactors = F)
rownames(phenoData) <- phenoData$Sample_Name
phenoData_meth_RNA <- phenoData[which(phenoData$RNA_expression == "YES" & (phenoData$Group == "B-PROGENITORS" | phenoData$Group == "MLL-AF4" | phenoData$Group == "MLL-AF9" | phenoData$Group == "MLL GERMLINE")),]

bVals <- read.csv("./Tables/bVals.txt", header = T, sep = "\t", stringsAsFactors = F)
bVals <- bVals[which(rownames(bVals) %in% rownames(EPIC.manifest)), phenoData_meth_RNA$Sample_Name]
bVals <- bVals[complete.cases(bVals),]
bVals <- as.matrix(bVals)

RNAseq <- read.table(file = "RSEM_RNAseq_genes_FPKM_matrix.tsv", header = T, sep = "\t")
colnames(RNAseq)[43:47] <- c("Normal_1", "Normal_2", "Normal_3", "Normal_4", "Normal_5")
RNAseq$MA4_01 <- as.numeric(as.character(RNAseq$MA4_01))
RNAseq <- RNAseq[!duplicated(RNAseq$ENSG_name), ]
RNAseq <- RNAseq[complete.cases(RNAseq),]
rownames(RNAseq) <- RNAseq$ENSG_name
RNAseq <- RNAseq[,c(-1,-2)]
RNAseq <- RNAseq + 1
RNAseq <- log(RNAseq,2)
RNAseq <- RNAseq[is.finite(rowSums(RNAseq)),]
keep <- rowSums(RNAseq) >= 20
RNAseq <- RNAseq[keep,]
RNAseq <- RNAseq[complete.cases(RNAseq),]
RNAseq <- RNAseq[, phenoData_meth_RNA$COD_ANT]
colnames(RNAseq) <- phenoData_meth_RNA$Sample_Name

AF4_probes <- read.csv(file = paste("DMPs_filtered_AF4.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
AF9_probes <- read.csv(file = paste(tablesDirectory, "DMPs_filtered_AF9.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
GERM_probes <- read.csv(file = paste(tablesDirectory, "DMPs_filtered_GERMLINE.txt", sep = "/"), header = T, sep = "\t", stringsAsFactors = F)
probes <- as.character(unique(c(rownames(AF4_probes), rownames(AF9_probes), rownames(GERM_probes))))

get.probes.motif <- function (data, probes.motif, probes, min.motif.quality = "DS", 
                              background.probes, pvalue = 0.05, lower.OR = 1.1, min.incidence = 10, 
                              dir.out = "./", label = NULL, save = TRUE, plot.title = NULL) 
{
  if (missing(probes.motif)) {
    if (missing(data)) 
      stop("Please set probes.motif argument. See ELMER data")
    file <- paste0("Probes.motif.", metadata(data)$genome, 
                   ".", metadata(data)$met.platform)
    message("Loading object: ", file)
    if (file == "Probes.motif.hg38.450K") 
      probes.motif <- ELMER:::getdata("Probes.motif.hg38.450K")
    if (file == "Probes.motif.hg19.450K") 
      probes.motif <- ELMER:::getdata("Probes.motif.hg19.450K")
    if (file == "Probes.motif.hg38.EPIC") 
      probes.motif <- ELMER:::getdata("Probes.motif.hg38.EPIC")
    if (file == "Probes.motif.hg19.EPIC") 
      probes.motif <- ELMER:::getdata("Probes.motif.hg19.EPIC")
  }
  
  all.probes.TF <- probes.motif
  if (missing(probes)) 
    stop("probes option should be specified.")
  if (missing(background.probes)) {
    if (!missing(data)) {
      background.probes <- as.character(names(getMet(data)))
    }
  }
  probes <- unique(probes)
  background.probes <- background.probes[background.probes %in% 
                                           rownames(all.probes.TF)]
  bg.probes.TF <- all.probes.TF[background.probes, ]
  bg.Probes.TF.percent <- Matrix::colMeans(bg.probes.TF)
  probes.TF <- all.probes.TF[rownames(all.probes.TF) %in% 
                               probes, ]
  probes.TF.num <- Matrix::colSums(probes.TF, na.rm = TRUE)
  p <- Matrix::colMeans(probes.TF)
  P <- bg.Probes.TF.percent
  sub.enrich.TF <- multiply_by(p, (1 - P)) %>% divide_by(P) %>% 
    divide_by(1 - p)
  sub.enrich.TF[is.nan(sub.enrich.TF)] <- 0
  a <- Matrix::colSums(probes.TF)
  b <- nrow(probes.TF) - Matrix::colSums(probes.TF)
  c <- Matrix::colSums(bg.probes.TF)
  d <- nrow(bg.probes.TF) - Matrix::colSums(bg.probes.TF)
  fisher <- plyr::adply(seq_len(length(a)), .margins = 1, 
                        .fun = function(i) {
                          x <- fisher.test(matrix(c(a[i], b[i], c[i], d[i]), 
                                                  nrow = 2, ncol = 2))
                          ret <- data.frame(x$conf.int[1], x$conf.int[2], 
                                            x$estimate, x$p.value)
                          colnames(ret) <- c("lowerOR", "upperOR", "OR", "p.value")
                          ret
                        }, .id = NULL, .progress = "text")
  rownames(fisher) <- names(a)
  Summary <- data.frame(motif = names(a), NumOfProbes = probes.TF.num, 
                        PercentageOfProbes = probes.TF.num/length(probes), fisher, 
                        FDR = p.adjust(fisher$p.value, method = "BH"), stringsAsFactors = FALSE)
  hocomoco <- ELMER:::getHocomocoTable()
  family.class <- hocomoco[, c("Model", grep("family", colnames(hocomoco), 
                                             value = T))]
  Summary <- merge(Summary, family.class, by.x = "motif", 
                   by.y = "Model")
  Summary <- Summary[order(Summary$lowerOR, decreasing = TRUE), 
                     ]
  en.motifs <- as.character(Summary[, "motif"])
  ELMER:::print.header("Filtering motifs based on quality", "subsection")
  message("Number of enriched motifs with quality:")
  message("-----------")
  for (q in c("A", "B", "C", "D", "S")) message(paste0(" => ", 
                                                       q, ": ", length(grep(paste0("\\.", q), en.motifs))))
  message("-----------")
  en.motifs <- grep(paste0("\\.[A-", toupper(min.motif.quality), 
                           "]"), en.motifs, value = T)
  message("Considering only motifs with quality from A up to ", 
          min.motif.quality, ": ", length(en.motifs), " motifs are enriched.")
  enriched.motif <- alply(en.motifs, function(x, probes.TF) {
    rownames(probes.TF[probes.TF[, x] == 1, x, drop = FALSE])
  }, probes.TF = probes.TF, .margins = 1, .dims = FALSE)
  attributes(enriched.motif) <- NULL
  names(enriched.motif) <- en.motifs
  
  return(enriched.motif)
}

mae <- get(load("./mae_ALL_conditions.rda"))
enriched.motif <- get.probes.motif(data = mae, probes = probes, min.incidence = 10, lower.OR = 1.1, background.probes = rownames(bVals), min.motif.quality = c("BS"), save = FALSE, label = "ALL_probes")

plot.average.meth.motif <- function(TF, motif.name, alias){
    selected_probes <- enriched.motif[[motif.name]]
    bVals_filtered <- bVals[selected_probes,]
    bVals_filtered_mean <- colMeans(bVals_filtered)
    correlation <- cor(as.numeric(as.vector(bVals_filtered_mean)), as.numeric(as.vector(RNAseq[which(rownames(RNAseq) == TF),])), method = "pearson")
    correlationmatrix <- data.frame(valuesx = as.numeric(as.vector(bVals_filtered_mean)), valuesy = as.numeric(RNAseq[which(rownames(RNAseq) == TF),]), group = c(phenoData_meth_RNA$Group), color = c(phenoData_meth_RNA$color))
    ggplot(correlationmatrix, aes(x=valuesx, y=valuesy)) + geom_point(size = 3, colour = correlationmatrix$color, alpha = .8) + geom_smooth(method = 'lm',size = 1, colour = 'black', se = F) + annotate("text", label = paste("Cor = ", round(correlation,2)), x = Inf, y = Inf, size = 6, colour = "black", vjust = 2, hjust = 1.1) + labs(x = "Average mean motif B-Value", y = "Gene Expression (Log2 RSEM FPKM)") + theme(aspect.ratio=1) + theme_bw() + ggtitle(label = paste(motif.name, "-", alias))
  }  

pdf(file="average_TF_methplot_FOS_JUN_E2F5.pdf", height = 4, width = 5, onefile = TRUE)
plot.average.meth.motif("ENSG00000159216", "RUNX1_HUMAN.H11MO.0.A", "RUNX1")
plot.average.meth.motif("ENSG00000133740", "E2F5_HUMAN.H11MO.0.B", "E2F5")
plot.average.meth.motif("ENSG00000075426", "FOSL2_HUMAN.H11MO.0.A", "FOSL2")
dev.off()

plot.average.meth.motif("ENSG00000170345", "FOS_HUMAN.H11MO.0.A", "FOS")
plot.average.meth.motif("ENSG00000177606", "JUN_HUMAN.H11MO.0.A", "JUN")

# Load the data:
RNAseq <- read.table(file = "Gene_expression/RSEM_RNAseq_genes_FPKM_matrix.tsv", header = T, sep = "\t")
colnames(RNAseq)[43:47] <- c("Normal_1", "Normal_2", "Normal_3", "Normal_4", "Normal_5")
RNAseq$MA4_01 <- as.numeric(as.character(RNAseq$MA4_01))
RNAseq <- RNAseq[!duplicated(RNAseq$ENSG_name), ]
RNAseq <- RNAseq[complete.cases(RNAseq),]
rownames(RNAseq) <- RNAseq$ENSG_name
RNAseq <- RNAseq[,c(-1,-2)]
RNAseq <- RNAseq + 1
RNAseq <- log(RNAseq,2)
RNAseq <- RNAseq[is.finite(rowSums(RNAseq)),]
keep <- rowSums(RNAseq) >= 20
RNAseq <- RNAseq[keep,]
RNAseq <- RNAseq[complete.cases(RNAseq),]
RNAseq$mean_AF4 <- rowMeans(RNAseq[,c(1:27)])
RNAseq$mean_AF9 <- rowMeans(RNAseq[,c(28:32)])
RNAseq$mean_GERM <- rowMeans(RNAseq[,c(33:40)])
RNAseq$mean_ProB <- rowMeans(RNAseq[,c(41:45)])
RNAseq$mean_B <- rowMeans(RNAseq[,c(46:48)])
RNAseq$fold_AF4 <- log((RNAseq$mean_AF4 / RNAseq$mean_ProB),2)
RNAseq$fold_AF9 <- log((RNAseq$mean_AF9 / RNAseq$mean_ProB),2)
RNAseq$fold_GERM <- log((RNAseq$mean_GERM / RNAseq$mean_ProB),2)
RNAseq$fold_B <- log((RNAseq$mean_B / RNAseq$mean_ProB),2)
RNAseq <- RNAseq[is.finite(rowSums(RNAseq)),]

COMMON <- read.csv("filtered_pairs_COMMON_new_2.txt", sep = "\t", stringsAsFactors = F)
AF4 <- read.csv("filtered_pairs_AF4_new_2.txt", sep = "\t", stringsAsFactors = F)
AF9 <- read.csv("filtered_pairs_AF9_new_2.txt", sep = "\t", stringsAsFactors = F)
GERM <- read.csv("filtered_pairs_GERM_new_2.txt", sep = "\t", stringsAsFactors = F)

############### Add probes which are present in the corresponding conditions:

DMPs_AF4 <- read.csv("~/Documents/Infant_leukemia/Tables/DMPs_filtered_AF4.txt", sep = "\t", stringsAsFactors = F)
DMPs_AF9 <- read.csv("~/Documents/Infant_leukemia/Tables/DMPs_filtered_AF9.txt", sep = "\t", stringsAsFactors = F)
DMPs_GERM <- read.csv("~/Documents/Infant_leukemia/Tables/DMPs_filtered_GERMLINE.txt", sep = "\t", stringsAsFactors = F)
all_entries_AF4 <- rbind(COMMON[which(COMMON$Probe %in% rownames(DMPs_AF4)),], AF4)
all_entries_AF9 <- rbind(COMMON[which(COMMON$Probe %in% rownames(DMPs_AF9)),], AF9)
all_entries_GERM <- rbind(COMMON[which(COMMON$Probe %in% rownames(DMPs_GERM)),], GERM)

# Get selected genes:
get.map.probes.average.meth.motif <- function(motif.name, alias){
  selected_probes <- enriched.motif[[motif.name]]
  all_entries_AF4_filtered <- all_entries_AF4[which(all_entries_AF4$Probe %in% selected_probes),]
  all_entries_AF9_filtered <- all_entries_AF9[which(all_entries_AF9$Probe %in% selected_probes),]
  all_entries_GERM_filtered <- all_entries_GERM[which(all_entries_GERM$Probe %in% selected_probes),]
  RNAseq_filtered_AF4 <- RNAseq[which(rownames(RNAseq) %in% unique(all_entries_AF4_filtered$GeneID)),]
  RNAseq_filtered_AF9 <- RNAseq[which(rownames(RNAseq) %in% unique(all_entries_AF9_filtered$GeneID)),]
  RNAseq_filtered_GERM <- RNAseq[which(rownames(RNAseq) %in% unique(all_entries_GERM_filtered$GeneID)),]
  RNAseq_filtered_B <- RNAseq[which(rownames(RNAseq) %in% unique(c(rownames(RNAseq_filtered_AF4), rownames(RNAseq_filtered_AF9), rownames(RNAseq_filtered_GERM)))),]
  AF4_genes <- data.frame(Group = c(rep("MLL-AF4", nrow(RNAseq_filtered_AF4))), Value = RNAseq_filtered_AF4$fold_AF4)
  AF9_genes <- data.frame(Group = c(rep("MLL-AF9", nrow(RNAseq_filtered_AF9))), Value = RNAseq_filtered_AF9$fold_AF9)
  GERM_genes <- data.frame(Group = c(rep("non-MLL", nrow(RNAseq_filtered_GERM))), Value = RNAseq_filtered_GERM$fold_GERM)
  B_genes <- data.frame(Group = c(rep("rest_Bcell", nrow(RNAseq_filtered_B))), Value = RNAseq_filtered_B$fold_B)
  sampRandomgenes <- sample(rownames(RNAseq),nrow(RNAseq_filtered_B))
  Random_genes <- RNAseq[sampRandomgenes,]
  Random_genes <- data.frame(Group = c(rep("Random", nrow(RNAseq_filtered_B)*4)), Value = c(Random_genes$fold_AF4, Random_genes$fold_AF9, Random_genes$fold_GERM, Random_genes$fold_B))
  genes <- rbind(AF4_genes, AF9_genes, GERM_genes, B_genes, Random_genes)
  ggplot(genes, aes(x=Group, y=Value, fill=Group)) + geom_violin(trim = FALSE) + geom_boxplot(width = 0.1, fill="white", outlier.shape = NA) + scale_fill_manual(values=c("#D53E4F", "#2eaa6c", "#a59c31", "#763d91", "#EBEBEB")) + labs(x = "Subtype", y = "Average fold change motif targets") + theme(aspect.ratio=1, legend.position="none") + theme_bw() + ggtitle(label = alias)
}

pdf(file="average_exp_targets_motifs_specific_group.pdf", height = 4, width = 5, onefile = TRUE)
get.map.probes.average.meth.motif("RUNX1_HUMAN.H11MO.0.A", "RUNX1")
get.map.probes.average.meth.motif("E2F5_HUMAN.H11MO.0.B", "E2F5")
get.map.probes.average.meth.motif("FOSL2_HUMAN.H11MO.0.A", "FOSL2")
dev.off()




# Get specific genes for network representation:
get.filtered.probe.pairs.motif <- function(motif.name, alias, all_entries){
  selected_probes <- enriched.motif[[motif.name]]
  all_entries_filtered <- all_entries[which(all_entries$Probe %in% selected_probes),]
  return(all_entries_filtered)
  }

AF4_RUNX1 <- get.filtered.probe.pairs.motif("RUNX1_HUMAN.H11MO.0.A", "RUNX1", all_entries_AF4)
AF4_FOSL2 <- get.filtered.probe.pairs.motif("FOSL2_HUMAN.H11MO.0.A", "FOSL2", all_entries_AF4)
AF4_E2F5 <- get.filtered.probe.pairs.motif("E2F5_HUMAN.H11MO.0.B", "E2F5", all_entries_AF4)
AF4_FOS <- get.filtered.probe.pairs.motif("FOS_HUMAN.H11MO.0.A", "FOS", all_entries_AF4)
AF4_JUN <- get.filtered.probe.pairs.motif("JUN_HUMAN.H11MO.0.A", "JUN", all_entries_AF4)


library("igraph") 
library("network") 
library("sna")
library("visNetwork")
library("threejs")
library("networkD3")
library("ndtv")
library("reshape2")
library("RColorBrewer")

generate.network <- function(interaction_table, perc.nodes, size.genes, size.cpgs, ylim, xlim){

    tab <- table(interaction_table$Symbol)
    estimate_for_representation <- round(max(tab)*perc.nodes/100)
    interaction_table <- interaction_table[interaction_table$Symbol %in% names(tab[tab > estimate_for_representation]),]
    interaction_edges <- as.data.frame(xtabs(~Symbol+Probe,interaction_table))
    interaction_edges$type <- "hyperlink"
    colnames(interaction_edges) <- c("from", "to", "weight", "type")
    interaction_edges <- interaction_edges[which(interaction_edges$weight == 1 | interaction_edges$weight == 2 | interaction_edges$weight == 3),]

    interaction_nodes <- data.frame(id = c(unique(interaction_table$Symbol), unique(interaction_table$Probe)), class = c(rep("Gene", length(unique(interaction_table$Symbol))), rep("CpG", length(unique(interaction_table$Probe)))), meth = c(rep(NA, length(unique(interaction_table$Symbol))), unique(interaction_table$LogFC_meth_LEUKEMIA.vs.B.PROGENITORS)))
    interaction_nodes$ge <- interaction_table$log2FC_GE_LEUKEMIA.vs.B.PROGENITORS[match(interaction_nodes$id, interaction_table$Symbol)]
    interaction_nodes$id <- as.character(interaction_nodes$id)
    nrow(interaction_nodes); length(unique(interaction_nodes$id))
    nrow(interaction_edges); nrow(unique(interaction_edges[,c("from", "to")]))

    net <- graph_from_data_frame(d=interaction_edges, vertices=interaction_nodes, directed=F) 

    colors_meth <- NA
    colors_meth[interaction_nodes$meth[complete.cases(interaction_nodes$meth)] >= 0] <- "yellow" 
    colors_meth[interaction_nodes$meth[complete.cases(interaction_nodes$meth)] <= 0] <- "skyblue" 

    colors_ge <- NA
    colors_ge[interaction_nodes$ge[complete.cases(interaction_nodes$ge)] >= 0] <- "#E5232F" 
    colors_ge[interaction_nodes$ge[complete.cases(interaction_nodes$ge)] <= 0] <- "#6057A5" 

    V(net)$color <- c(colors_ge, colors_meth)
    deg <- centralization.degree(net)$res
    end1 <- nrow(interaction_nodes[which(interaction_nodes$class == "Gene"),])
    start2 <- end1 + 1
    end2 <- nrow(interaction_nodes)   
    V(net)$size[1:end1]<- deg[1:end1]*size.genes
    V(net)$size[start2:end2]<- deg[start2:end2]*size.cpgs
    V(net)$label.cex = 0.5
    V(net)$dist[1:end1]<- 1
    V(net)$dist[start2:end2]<- 6

    l <- layout_with_fr(net)
    plot(net, layout=l, rescale = FALSE, edge.arrow.size=5, edge.curved=.1, ylim=ylim, xlim=xlim, vertex.label.dist=V(net)$dist)
}

pdf(file="network_correlated_genes_CpGs.pdf", family="Garamond", paper = "a4r", height = 21, width = 28, onefile = TRUE)
generate.network(AF4_RUNX1, 30, 30, 25, ylim=c(-5,25),xlim=c(0,20))
generate.network(AF4_FOSL2, 15, 30, 25, ylim=c(-5,25),xlim=c(0,20))
generate.network(AF4_E2F5, 40, 30, 25, ylim=c(-5,25),xlim=c(0,20))

generate.network(AF4_FOS, 15, 30, 25, ylim=c(-5,25),xlim=c(0,20))
generate.network(AF4_JUN, 15, 30, 25, ylim=c(-5,25),xlim=c(0,20))

interaction_table <- AF4_FOSL2
interaction_table$gedir[interaction_table$log2FC_GE_LEUKEMIA.vs.B.PROGENITORS > 0] <- "up"
interaction_table$gedir[interaction_table$log2FC_GE_LEUKEMIA.vs.B.PROGENITORS < 0] <- "down"
ggplot(interaction_table, aes(x=factor(gedir))) + geom_bar(stat="count") + theme_bw()
dev.off()

write.table(AF4_E2F5, file = "AF4_E2F5_motif_targets.txt", sep = "\t", col.names = T, row.names = F, quote = F)
write.table(AF4_FOSL2, file = "AF4_FOSL2_motif_targets.txt", sep = "\t", col.names = T, row.names = F, quote = F)
write.table(AF4_RUNX1, file = "AF4_RUNX1_motif_targets.txt", sep = "\t", col.names = T, row.names = F, quote = F)

