#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script for the generation of plots and figures
###          
###
### Authors: Juan Ramon Tejedor (Dec 2020)
#############################################################################

# Plot graphs for revision:

library(ggplot2)
library(tidyr)
library(reshape2)
library(plyr)
library(genefilter)
library(pheatmap)
library(RColorBrewer)
library(UpSetR)
library(factoextra)
library(ggrepel)
library(eulerr)
library(ELMER.data)
library(annotatr)
library(ggpubr)
library(dplyr)

# Set up the directory for the results:
resultsDirectory <- ""
tablesDirectory <- ""

# Load datasets:
bVals <- read.csv("bVals_revision.txt", sep = "\t", stringsAsFactors = F)
data("Probes.motif.hg19.EPIC")
Motifs_1 <- as.data.frame(Probes.motif.hg19.EPIC[,c("FOSL2_HUMAN.H11MO.0.A")])
bVals_FOSL2motifs <- bVals[Motifs1$Name,]

# plot scatterplots:
bVals_AF4_FLC <- bVals[,1:2]
bVals_AF4_FLC$Sig <- "NO"
bVals_AF4_FLC$Sig[which(abs(bVals_AF4_FLC$AF4 - bVals_AF4_FLC$FLC) > 0.2)] <- "YES"

pdf("Scatterplot_revision.pdf", height = 7, width = 7)

ggplot(bVals_AF4_FLC, aes(FLC, AF4, color = Sig, alpha = 0.5)) +
  geom_point(shape = 16, size = 1, show.legend = FALSE) +
  scale_color_manual(values = c("#ECECEC", "#31688E")) +
  geom_smooth(method = "lm", se = FALSE, linetype = "dashed", color = "grey30") +
  stat_cor(method = "spearman", alternative = "two.sided") +
  geom_abline(intercept = c(.2, -.2), slope = 1, color = c("red", "red"), linetype = "dashed") +
  xlab(label = "FLC") + ylab(label = "AF4trans") + 
  ggtitle(label = "Correlation Plot FLC/AF4trans") +
  theme_bw() +
  theme(legend.position = "none", aspect.ratio=1)
  
dev.off()


# Plot correlations between factors:

GTRD_hyper_AF4trans <- read.csv("enrichment_GTRD_tfbs_hyper_AF4_vs_FLC_revision.txt", sep = "\t", stringsAsFactors = F)
GTRD_hypo_AF4trans <- read.csv("enrichment_GTRD_tfbs_hypo_AF4_vs_FLC_revision.txt", sep = "\t", stringsAsFactors = F)

GTRD_hyper_AF4trans$logQval <- -log(GTRD_hyper_AF4trans$qValue, 10)
GTRD_hypo_AF4trans$logQval <- -log(GTRD_hypo_AF4trans$qValue, 10)

GTRD_hyper_AF4trans$number <- c(1:681)
GTRD_hyper_AF4trans$number <- gsub(pattern = "^", replacement = "#", x = GTRD_hyper_AF4trans$number)
GTRD_hypo_AF4trans$number <- c(1:681)
GTRD_hypo_AF4trans$number <- gsub(pattern = "^", replacement = "#", x = GTRD_hypo_AF4trans$number)

GTRD_hyper_AF4trans$description <- gsub(pattern = "GTRD_tfbs ", replacement = "", x = GTRD_hyper_AF4trans$description)
GTRD_hypo_AF4trans$description <- gsub(pattern = "GTRD_tfbs ", replacement = "", x = GTRD_hypo_AF4trans$description)

fosfamily <- c("FosB", "Fra-1", "JunB", "c-Fos", "Fra-2", "JunD")

GTRD_hyper_AF4trans$Fosfam <- "NO"
GTRD_hyper_AF4trans$Sig <- "NO"
GTRD_hyper_AF4trans$Fosfam[which(GTRD_hyper_AF4trans$description %in% fosfamily)] <- "YES"
GTRD_hyper_AF4trans$Sig[which(GTRD_hyper_AF4trans$qValue < 0.05)] <- "YES"

GTRD_hypo_AF4trans$Fosfam <- "NO"
GTRD_hypo_AF4trans$Sig <- "NO"
GTRD_hypo_AF4trans$Fosfam[which(GTRD_hypo_AF4trans$description %in% fosfamily)] <- "YES"
GTRD_hypo_AF4trans$Sig[which(GTRD_hypo_AF4trans$qValue < 0.05)] <- "YES"

GTRD_hyper_AF4trans <- GTRD_hyper_AF4trans[1:30,]
GTRD_hypo_AF4trans <- GTRD_hypo_AF4trans[1:30,]

pdf(file = paste(tablesDirectory, "Enrichment_motifs_AF4trans_revision.pdf", sep = ""), paper = "a4r", height = 21, width = 28, onefile = T)

ggplot(GTRD_hyper_AF4trans, aes(x = reorder(description, -logQval), y = logQval, color = Fosfam, label = description)) +
  geom_point(show.legend = T, size = 4) +
  scale_color_manual(values = c("grey50","#BC72AF")) + 
  geom_text_repel(data = subset(GTRD_hyper_AF4trans, Sig == "YES" | Fosfam == "YES"), nudge_x = 0, nudge_y = 0, point.padding = 0.5, force = 1) +
  xlab(label = "Sample Name") + ylab(label = "Enrichment Score (-Log10 pval)") + 
  ggtitle(label = "TFBS enrichment hyper AF4trans") +
  theme_bw() +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1), legend.position = "none", aspect.ratio=0.25)

ggplot(GTRD_hypo_AF4trans, aes(x = reorder(description, -logQval), y = logQval, color = Fosfam, label = description)) +
  geom_point(show.legend = T, size = 4) +
  scale_color_manual(values = c("grey50","#BC72AF")) + 
  geom_text_repel(data = subset(GTRD_hypo_AF4trans, Sig == "YES" | Fosfam == "YES"), nudge_x = 0, nudge_y = 0, point.padding = 0.5, force = 1) +
  xlab(label = "Sample Name") + ylab(label = "Enrichment Score (-Log10 pval)") + 
  ggtitle(label = "TFBS enrichment hypo AF4trans") +
  theme_bw() +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1), legend.position = "none", aspect.ratio=0.25)

dev.off()

# Plot barplot enrichments:
homer <- read.csv("HOMER/FOSL2KO_hyper/knownResults.txt", sep = "\t", stringsAsFactors = F)
homer <- homer[1:20,]

pdf(file = paste(tablesDirectory, "Enrichment_motifs_FOSL2KO_revision.pdf", sep = ""), paper = "a4r", height = 21, width = 28, onefile = T)

ggplot(homer, aes(x = reorder(Motif.Name, Log.P.value), y = -Log.P.value, fill = -Log.P.value)) +
  geom_bar(show.legend = T, width = 0.5, stat = "identity") +
  scale_fill_gradient(low = "grey50", high = "#BC72AF") + 
  xlab(label = "Sample Name") + ylab(label = "Enrichment Score (-Log10 pval)") + 
  ggtitle(label = "HOMER enrichment hyper FOSL2KO") +
  theme_bw() +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1), legend.position = "none", aspect.ratio=0.25)

dev.off()

library(ggplot2)
library(gridExtra)
library(genefilter)
library(tidyr)
library(ggbio)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(IlluminaHumanMethylationEPICanno.ilm10b2.hg19)
library(grid)
library(lattice)
library(reshape2)
library(biomaRt)


# Plot Meth data for selected genes:
bVals <- read.csv("bVals_CRISPR_exp.txt", sep = "\t", stringsAsFactors = F)
bVals_FOSL2 <- bVals[,3:4]
bVals_FOSL2$FOSL2_vs_SEM <- bVals_FOSL2$FOSL2 - bVals_FOSL2$SEM

bVals_leukemia <- read.csv(bVals.txt", sep = "\t", stringsAsFactors = F)
bVals_leukemia$AF4 <- rowMeans(bVals_leukemia[,1:37])
bVals_leukemia <- bVals_leukemia[rownames(bVals_FOSL2),]
bVals_leukemia <- bVals_leukemia[complete.cases(bVals_leukemia),]
bVals_FOSL2 <- bVals_FOSL2[rownames(bVals_leukemia),]
bVals_FOSL2$AF4 <- bVals_leukemia$AF4

DMPs_FOSL2 <- bVals_FOSL2[which(abs(bVals_FOSL2$FOSL2_vs_SEM) > 0.4),]
DMPs_FOSL2$Dir <- "hypo"
DMPs_FOSL2$Dir[which(DMPs_FOSL2$FOSL2_vs_SEM > 0)] <- "hyper" 

# Calculate methylation
annotationEPIC <- as.data.frame(getAnnotation(IlluminaHumanMethylationEPICanno.ilm10b2.hg19))
txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
annotationEPIC <- separate(data = annotationEPIC, col = "GencodeCompV12_NAME", into = c("genesymbol", "other_gene_symbol"), sep = ";", remove = T)
annotationEPIC$chr2 <- gsub(pattern = "chr", replacement = "", x = annotationEPIC$chr)
annotationEPIC$end <- annotationEPIC$pos + 1
rownames(annotationEPIC) <- annotationEPIC$Name
annotationEPIC$strand <- "+"
annotationEPIC <- annotationEPIC[rownames(bVals_FOSL2),]
annotationEPIC <- annotationEPIC[order(annotationEPIC$chr2, annotationEPIC$pos),]
colnames(annotationEPIC)[2] <- "start"
annotationEPIC <- annotationEPIC[,c("chr", "start", "end", "strand", "Name")]
bVals_media <- bVals_FOSL2[rownames(annotationEPIC),]
bVals_media$SEM <- bVals_media$SEM
bVals_media$FOSL2 <- bVals_media$FOSL2
bVals_media$AF4 <- bVals_media$AF4
bVals_media <- bVals_media[, c("SEM", "FOSL2", "AF4")]
bVals_media$Significant <- FALSE
bVals_media$Significant[which(abs(bVals_media$FOSL2 - bVals_media$SEM) > 0.4)] <- TRUE

source("plotRangesLinkedToData.R")

# # Represent Meth plot for a given gene:
methylationplot <- function(geneofinterest, chromosome, start, end){
  annotationEPIC_filtered <- annotationEPIC[which(annotationEPIC$chr == chromosome & annotationEPIC$start >= start & annotationEPIC$end <= end),]
  bVals_media <- bVals_media[rownames(annotationEPIC_filtered),]
  annotationEPIC_filtered2 <- annotationEPIC_filtered
  rownames(annotationEPIC_filtered2) <- c(1:length(annotationEPIC_filtered2[,1]))
  cpgs.new_5mC <- makeGRangesFromDataFrame(annotationEPIC_filtered2, keep.extra.columns = F)
  values(cpgs.new_5mC)$SEM <- bVals_media$SEM
  values(cpgs.new_5mC)$FOSL2 <- bVals_media$FOSL2
  values(cpgs.new_5mC)$AF4 <- bVals_media$AF4
  values(cpgs.new_5mC)$Name <- annotationEPIC_filtered2$Name
  values(cpgs.new_5mC)$Significant <- bVals_media$Significant
  gene <- GRanges(chromosome, IRanges(start, end))
  genecoords <- autoplot(txdb, gene, fill = "brown", color = "brown")
  plotRangesLinkedToData(cpgs.new_5mC, stat.y = c(1:3), stat.ylab = paste("Meth - ", geneofinterest, sep  = ""), size = 1, sig = "Significant", sig.col = c("gray70", "red"), theme.stat = theme_bw(), annotation = genecoords, height = c(200, 1, 1, 1), plot = T, ylim(c(0,1)))
}

pdf(file = paste(tablesDirectory, "Methplot_selected_genes_revision.pdf", sep = ""), paper = "a4r", height = 21, width = 28, onefile = T)

methylationplot(geneofinterest = "NUDT21", chromosome = "chr16", start = 56460437, end = 56505101)
methylationplot(geneofinterest = "FSCN1", chromosome = "chr7", start = 5630844, end = 5649352)

dev.off()


