#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script for the analysis of RNAseq data
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

resultsDirectory <- ""
tablesDirectory <- ""

library(EBSeq)

# Load the datasets:
rnaseq <- data.matrix(read.table(file= paste(tablesDirectory, "RSEM_RNAseq_genes_counts_matrix.csv", sep = "/")))
phenodata <- read.csv(file= paste(tablesDirectory, "phenoData.csv", sep = "/"), sep = "\t", stringsAsFactors = F)
colnames(rnaseq) <- phenodata$Sample_Name_RNAseq

phenodata_AF4 <- phenodata[which(phenodata$Sample_Group == "MLLAF4" | phenodata$Sample_Group == "ProB_fetal"),]
phenodata_AF4$Sample_Group <- factor(phenodata_AF4$Sample_Group, levels = c("MLLAF4", "ProB_fetal"))
rnaseq_AF4 <- rnaseq[,phenodata_AF4$Sample_Name_RNAseq]

phenodata_AF9 <- phenodata[which(phenodata$Sample_Group == "MLLAF9" | phenodata$Sample_Group == "ProB_fetal"),]
phenodata_AF9$Sample_Group <- factor(phenodata_AF9$Sample_Group, levels = c("MLLAF9", "ProB_fetal"))
rnaseq_AF9 <- rnaseq[,phenodata_AF9$Sample_Name_RNAseq]

phenodata_WT <- phenodata[which(phenodata$Sample_Group == "MLLWT" | phenodata$Sample_Group == "ProB_fetal"),]
phenodata_WT$Sample_Group <- factor(phenodata_WT$Sample_Group, levels = c("MLLWT", "ProB_fetal"))
rnaseq_WT <- rnaseq[,phenodata_WT$Sample_Name_RNAseq]

phenodata_B <- phenodata[which(phenodata$Sample_Group == "Bcell" | phenodata$Sample_Group == "ProB_fetal"),]
phenodata_B$Sample_Group <- factor(phenodata_B$Sample_Group, levels = c("Bcell", "ProB_fetal"))
rnaseq_B <- rnaseq[,phenodata_B$Sample_Name_RNAseq]

# Calculate library sizes factor for each sample:
Sizes_AF4 = MedianNorm(rnaseq_AF4)
Sizes_AF9 = MedianNorm(rnaseq_AF9)
Sizes_WT = MedianNorm(rnaseq_WT)
Sizes_B = MedianNorm(rnaseq_B)

# Calculate differentially expressed genes:
EBOut_AF4 = EBTest(Data = rnaseq_AF4, Conditions = phenodata_AF4$Sample_Group, sizeFactors = Sizes_AF4, maxround=5)
EBOut_AF9 = EBTest(Data = rnaseq_AF9, Conditions = phenodata_AF9$Sample_Group, sizeFactors = Sizes_AF9, maxround=5)
EBOut_WT = EBTest(Data = rnaseq_WT, Conditions = phenodata_WT$Sample_Group, sizeFactors = Sizes_WT, maxround=5)
EBOut_B = EBTest(Data = rnaseq_B, Conditions = phenodata_B$Sample_Group, sizeFactors = Sizes_B, maxround=5)

# Generate the tables for storing the data:
DEOut_AF4 <- GetDEResults(EBOut_AF4, FDR = 0.001)
FC_AF4 <- PostFC(EBOut_AF4)
res_AF4 <- data.frame(DEOut_AF4$PPMat[DEOut_AF4$DEfound,])
res_AF4$LogFC <- FC_AF4$PostFC[rownames(res_AF4)]
res_AF4$LogFC <- log2(res_AF4$LogFC)
res_AF4$Change <- NA
res_AF4$Change[which(res_AF4$LogFC > 0)] <- "UP" 
res_AF4$Change[which(res_AF4$LogFC < 0)] <- "DOWN"
res_AF4$Condition <- FC_AF4$Direction 

DEOut_AF9 <- GetDEResults(EBOut_AF9, FDR = 0.001)
FC_AF9 <- PostFC(EBOut_AF9)
res_AF9 <- data.frame(DEOut_AF9$PPMat[DEOut_AF9$DEfound,])
res_AF9$LogFC <- FC_AF9$PostFC[rownames(res_AF9)]
res_AF9$LogFC <- log2(res_AF9$LogFC)
res_AF9$Change <- NA
res_AF9$Change[which(res_AF9$LogFC > 0)] <- "UP" 
res_AF9$Change[which(res_AF9$LogFC < 0)] <- "DOWN"
res_AF9$Condition <- FC_AF9$Direction 

DEOut_WT <- GetDEResults(EBOut_WT, FDR = 0.001)
FC_WT <- PostFC(EBOut_WT)
res_WT <- data.frame(DEOut_WT$PPMat[DEOut_WT$DEfound,])
res_WT$LogFC <- FC_WT$PostFC[rownames(res_WT)]
res_WT$LogFC <- log2(res_WT$LogFC)
res_WT$Change <- NA
res_WT$Change[which(res_WT$LogFC > 0)] <- "UP" 
res_WT$Change[which(res_WT$LogFC < 0)] <- "DOWN"
res_WT$Condition <- FC_WT$Direction 

DEOut_B <- GetDEResults(EBOut_B, FDR = 0.001)
FC_B <- PostFC(EBOut_B)
res_B <- data.frame(DEOut_B$PPMat[DEOut_B$DEfound,])
res_B$LogFC <- FC_B$PostFC[rownames(res_B)]
res_B$LogFC <- log2(res_B$LogFC)
res_B$Change <- NA
res_B$Change[which(res_B$LogFC > 0)] <- "UP" 
res_B$Change[which(res_B$LogFC < 0)] <- "DOWN"
res_B$Condition <- FC_B$Direction 

# Write tables:
write.table(res_AF4, file = paste(tablesDirectory, "Ebseq_diff_exp_analysis_genes_AF4_vs_ProB_001FDR.tsv", sep = "/"), sep = "\t", quote = F, row.names = T, col.names = T)
write.table(res_AF9, file = paste(tablesDirectory, "Ebseq_diff_exp_analysis_genes_AF9_vs_ProB_001FDR.tsv", sep = "/"), sep = "\t", quote = F, row.names = T, col.names = T)
write.table(res_WT, file = paste(tablesDirectory, "Ebseq_diff_exp_analysis_genes_WT_vs_ProB_001FDR.tsv", sep = "/"), sep = "\t", quote = F, row.names = T, col.names = T)
write.table(res_B, file = paste(tablesDirectory, "Ebseq_diff_exp_analysis_genes_B_vs_ProB_001FDR.tsv", sep = "/"), sep = "\t", quote = F, row.names = T, col.names = T)
