#############################################################################
### PROJECT: Differential expression analysis of Infant leukemias
### and differentiated blood cells with RNAseq data
###
### Script for the calculation of gene ontologies
###          
###
### Authors: Juan Ramon Tejedor (Dec 2017)
#############################################################################

#################################################################################
###                 Calculate and represent gene ontologies                   ###
#################################################################################


# Download and install the required packages:
#install.packages("quadprog")
#source("http://bioconductor.org/biocLite.R")
#biocLite("IlluminaHumanMethylationEPICmanifest")
#biocLite("RColorBrewer")
#biocLite("IlluminaHumanMethylationEPICanno.ilm10b2.hg19")

# Load the required packages:
library("genefilter")
library("gplots")
library("cluster")
library("gridExtra")
library("factoextra")
library("RColorBrewer")
library("ggplot2")
library("reshape2")
library("dplyr")
library("tidyr")
library("TxDb.Hsapiens.UCSC.hg19.knownGene")
library("clusterProfiler")
library("ReactomePA")
library("org.Hs.eg.db")

options("width"=110) #  using R in terminal mode on Linux, making easier the inspection of data sets
###################################################################
#####                       READING DATA                      #####
###################################################################

# Set up the directory for the results:
hs <- org.Hs.eg.db
resultsDirectory <- ""
tablesDirectory <- ""
setwd(tablesDirectory)

COMMON <- read.csv("filtered_pairs_COMMON_new_2.txt", sep = "\t", stringsAsFactors = F)
AF4 <- read.csv("filtered_pairs_AF4_new_2.txt", sep = "\t", stringsAsFactors = F)
AF9 <- read.csv("filtered_pairs_AF9_new_2.txt", sep = "\t", stringsAsFactors = F)
GERM <- read.csv("filtered_pairs_GERM_new_2.txt", sep = "\t", stringsAsFactors = F)
Background <- read.csv("gene_expression.tsv", sep = "\t", stringsAsFactors = F)

COMMON_hyper <- unique(as.character(COMMON$Symbol[which(COMMON$Change_leukemia == "hyper")]))
COMMON_hyper <- select(hs, keys = COMMON_hyper, columns = c("ENTREZID", "SYMBOL"), keytype = "SYMBOL")
COMMON_hyper <- COMMON_hyper[complete.cases(COMMON_hyper),]
COMMON_hypo <- unique(as.character(COMMON$Symbol[which(COMMON$Change_leukemia == "hypo")]))
COMMON_hypo <- select(hs, keys = COMMON_hypo, columns = c("ENTREZID", "SYMBOL"), keytype = "SYMBOL")
COMMON_hypo <- COMMON_hypo[complete.cases(COMMON_hypo),]

AF4_hyper <- unique(as.character(AF4$Symbol[which(AF4$Change_leukemia == "hyper")]))
AF4_hyper <- select(hs, keys = AF4_hyper, columns = c("ENTREZID", "SYMBOL"), keytype = "SYMBOL")
AF4_hyper <- AF4_hyper[complete.cases(AF4_hyper),]
AF4_hypo <- unique(as.character(AF4$Symbol[which(AF4$Change_leukemia == "hypo")]))
AF4_hypo <- select(hs, keys = AF4_hypo, columns = c("ENTREZID", "SYMBOL"), keytype = "SYMBOL")
AF4_hypo <- AF4_hypo[complete.cases(AF4_hypo),]

AF9_hyper <- unique(as.character(AF9$Symbol[which(AF9$Change_leukemia == "hyper")]))
AF9_hyper <- select(hs, keys = AF9_hyper, columns = c("ENTREZID", "SYMBOL"), keytype = "SYMBOL")
AF9_hyper <- AF9_hyper[complete.cases(AF9_hyper),]
AF9_hypo <- unique(as.character(AF9$Symbol[which(AF9$Change_leukemia == "hypo")]))
AF9_hypo <- select(hs, keys = AF9_hypo, columns = c("ENTREZID", "SYMBOL"), keytype = "SYMBOL")
AF9_hypo <- AF9_hypo[complete.cases(AF9_hypo),]

GERM_hyper <- unique(as.character(GERM$Symbol[which(GERM$Change_leukemia == "hyper")]))
GERM_hyper <- select(hs, keys = GERM_hyper, columns = c("ENTREZID", "SYMBOL"), keytype = "SYMBOL")
GERM_hyper <- GERM_hyper[complete.cases(GERM_hyper),]
GERM_hypo <- unique(as.character(GERM$Symbol[which(GERM$Change_leukemia == "hypo")]))
GERM_hypo <- select(hs, keys = GERM_hypo, columns = c("ENTREZID", "SYMBOL"), keytype = "SYMBOL")
GERM_hypo <- GERM_hypo[complete.cases(GERM_hypo),]

Background <- unique(as.character(Background$GENE_name))
Background <- select(hs, keys = Background, columns = c("ENTREZID", "SYMBOL"), keytype = "SYMBOL")
Background <- Background[complete.cases(Background),]


# Load the data:
txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
genes = list(COMMON_hyper = COMMON_hyper$ENTREZID, COMMON_hypo = COMMON_hypo$ENTREZID, AF4_hyper = AF4_hyper$ENTREZID, AF4_hypo = AF4_hypo$ENTREZID, AF9_hyper = AF9_hyper$ENTREZID, AF9_hypo = AF9_hypo$ENTREZID, GERM_hyper = GERM_hyper$ENTREZID, GERM_hypo = GERM_hypo$ENTREZID)
# genes = list(COMMON = COMMON$ENTREZID, AF4 = AF4$ENTREZID, AF9 = AF9$ENTREZID, GERM = GERM$ENTREZID)

setwd(resultsDirectory)
pdf(file="Gene_kegg_ontologies_leukemias.pdf", paper = "a4r", height = 21, width = 28, onefile = TRUE)

compKEGG <- compareCluster(geneCluster = genes, 
                           fun           = "enrichKEGG",
                           organism      = "hsa",
                           pvalueCutoff  = 0.005,
                           universe = Background$ENTREZID,
                           pAdjustMethod = "BH")
plot(compKEGG, showCategory = 10, title = "KEGG Pathway Enrichment Analysis clusters")

compGO <- compareCluster(geneCluster   = genes,
                         fun           = "enrichGO",
                         OrgDb         = "org.Hs.eg.db",
                         ont           = "BP",
                         pvalueCutoff  = 0.0001,
                         qvalueCutoff = 0.0001,
                         pAdjustMethod = "BH",
                         universe = Background$ENTREZID)
plot(compGO, showCategory = 10, title = "GO Enrichment Analysis BP clusters")

compPathway <- compareCluster(geneCluster   = genes,
                              fun           = "enrichPathway",
                              pvalueCutoff  = 0.01,
                              qvalueCutoff = 0.01,
                              pAdjustMethod = "BH",
                              universe = Background$ENTREZID)
plot(compPathway, showCategory = 10, title = "Reactome Pathway Enrichment Analysis clusters")

dev.off()

enrich_COMMON_hypo <- enrichKEGG(gene = COMMON_hypo$ENTREZID, organism = "hsa", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_AF4_hypo <- enrichKEGG(gene = AF4_hypo$ENTREZID, organism = "hsa", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_AF9_hypo <- enrichKEGG(gene = AF9_hypo$ENTREZID, organism = "hsa", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_GERM_hypo <- enrichKEGG(gene = GERM_hypo$ENTREZID, organism = "hsa", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)

enrich_COMMON_hypo <- enrichGO(gene = COMMON_hypo$ENTREZID, OrgDb = "org.Hs.eg.db", ont = "BP", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_AF4_hypo <- enrichGO(gene = AF4_hypo$ENTREZID, OrgDb = "org.Hs.eg.db", ont = "BP", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_AF9_hypo <- enrichGO(gene = AF9_hypo$ENTREZID, OrgDb = "org.Hs.eg.db", ont = "BP", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_GERM_hypo <- enrichGO(gene = GERM_hypo$ENTREZID, OrgDb = "org.Hs.eg.db", ont = "BP", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)

enrich_COMMON_hyper <- enrichPathway(gene = COMMON_hyper$ENTREZID, organism = "human", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_AF4_hyper <- enrichPathway(gene = AF4_hyper$ENTREZID, organism = "human", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_AF9_hyper <- enrichPathway(gene = AF9_hyper$ENTREZID, organism = "human", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_GERM_hyper <- enrichPathway(gene = GERM_hyper$ENTREZID, organism = "human", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_COMMON_hypo <- enrichPathway(gene = COMMON_hypo$ENTREZID, organism = "human", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_AF4_hypo <- enrichPathway(gene = AF4_hypo$ENTREZID, organism = "human", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_AF9_hypo <- enrichPathway(gene = AF9_hypo$ENTREZID, organism = "human", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)
enrich_GERM_hypo <- enrichPathway(gene = GERM_hypo$ENTREZID, organism = "human", pvalueCutoff = 0.05, qvalueCutoff = 0.05, pAdjustMethod = "BH", universe = Background$ENTREZID)

emapplot(enrich_COMMON_hypo, categorySize="pvalue")
emapplot(enrich_AF4_hypo, categorySize="pvalue")
emapplot(enrich_AF9_hypo, categorySize="pvalue")
emapplot(enrich_GERM_hypo, categorySize="pvalue")
