#############################################################################
### PROJECT: Differential methylation analysis of Infant leukemias
### and differentiated blood cells with 850k arrays
###
### Script for the analysis of Methylation array data (850K)
###          
###
### Authors: Juan Ramon Tejedor (Dec 2020)
#############################################################################

# Download and install the required packages:
#install.packages("quadprog")
#source("http://bioconductor.org/biocLite.R")
#biocLite("limma")
#biocLite("minfi")
#biocLite("IlluminaHumanMethylationEPICmanifest")
#biocLite("IlluminaHumanMethylation450kmanifest")
#biocLite("RColorBrewer")
#biocLite("missMethyl") 
#biocLite("matrixStats")
#biocLite("Gviz") 
#biocLite("DMRcate") 
#biocLite("stringr")
#biocLite("IlluminaHumanMethylationEPICanno.ilm10b2.hg19")
#biocLite("IlluminaHumanMethylation450kanno.ilmn12.hg19")
#biocLite("GO.db")
#biocLite("FlowSorted.Blood.450k")
#biocLite("genefilter")
#biocLite("ChAMP")
#biocLite("sva")
#install.packages("FactoMineR")
#install.packages("devtools")
#install.packages("gplots")
#install.packages("cluster")
#install.packages("gridExtra")
#devtools::install_github("kassambara/factoextra")
#biocLite("ENmix")

# Load the required packages:
library("quadprog")
library("limma")
library("minfi")
library("IlluminaHumanMethylationEPICmanifest")
library("RColorBrewer")
library("missMethyl")
library("matrixStats")
library("Gviz")
library("DMRcate")
library("stringr")
library("IlluminaHumanMethylationEPICanno.ilm10b2.hg19")
library("GO.db")
library("genefilter")
library("ChAMP")
library("sva")
library("FactoMineR")
library("devtools")
library("gplots")
library("cluster")
library("gridExtra")
library("factoextra")
library("reshape2")
library("beanplot")
library("rgl")

source("~/Documents/ScriptsR/densityBeanPlot.R")

options("width"=110) #  using R in terminal mode on Linux, making easier the inspection of data sets
###################################################################
#####                       READING DATA                      #####
###################################################################

# Set up the path for the project and the IDATs location:
basedir <- ""
idatdir <- ""

# Set up the directory for the database filtering tables (multimapping, crossreactive, SNPs...):
dataDirectory <- ""

# Set up the directory for the results:
resultsDirectory <- ""
tablesDirectory <- ""

# Read the sample sheet for the experiment:
setwd(basedir)
targets <- read.metharray.sheet(paste(basedir, "samplesheets", sep = ""), pattern = ".csv$")

# Add the information of the slide array for further filtering purposes:
targets$SlideArray <- paste(targets$Slide, targets$Array, sep = "_")
targets$Basename <- paste(idatdir, targets$Slide, targets$SlideArray, sep = "/")

# Read the raw data from the IDAT files:
RGSet <- read.metharray.exp(targets = targets)

# Generate the phenodata for the desired dataset:
phenoData <- pData(RGSet)
phenoData$color <- c("#3184b7", "#3288BD", "#D53E4F", "#2eaa6c")

# Obtain probe design information from the dataset:
manifest <- getManifest(RGSet)

# Get annotation from RGSet (EPIC):
annotationEPIC <- getAnnotation(RGSet)

# Set the color palette for visual representations:
groups <- factor(phenoData$Sample_Group, levels = c("FLC", "AF4", "SEM", "FOSL2"))
pal <- factor(phenoData$color, levels = c("#3184b7", "#3288BD", "#D53E4F", "#2eaa6c"))

###################################################################
#####                    QUALITY CONTROL 1                    #####
###################################################################

# Calculate the detection p-values:
detP <- detectionP(RGSet)

# Examine mean detection p-values across all samples to identify any failed samples:
# "las=2" is for making label text perpendicular to axis
pdf(file= paste(resultsDirectory, "01-Qual_pvalue_plot_revision.pdf", sep = "/"), paper="a4r", height=21, width=28, onefile=TRUE)
par(mfrow=c(1,1))
barplot(colMeans(detP), col=phenoData$color, las=2, cex.axis=0.8, col.axis="grey40", cex.names=0.5, ylab="Mean detection p-values", names.arg=phenoData$Sample_Name, border=phenoData$color, space=1.5)
abline(h=0.01, col="green4")
legend("topright", legend=levels(groups), fill=levels(pal), bg="white")
barplot(colMeans(detP), col=phenoData$color, las=2, cex.axis=0.8, col.axis="grey40", cex.names=0.5, ylim=c(0,0.002), ylab="Mean detection p-values", names.arg=phenoData$Sample_Name, border=phenoData$color, space=1.5)
legend("topright", legend=levels(groups), fill=levels(pal), bg="white")
dev.off()

# Generate the QC report for the dataset:
qcReport(RGSet, sampNames=phenoData$Sample_Name, sampGroups = phenoData$Sample_Group, pdf= paste(resultsDirectory, "qcReport_revision.pdf", sep = "/"))

###################################################################
#####                    SAMPLE FILTERING                     #####
###################################################################

# Remove poor quality samples (i.e detP > 0.01 or detP > 0.05) and IVD:
keep <- colMeans(detP) < 0.01
RGSet_filtered <- RGSet[, keep]

# Remove poor quality samples from RGset, targets data, detP and phenoData:
targets_filtered <- targets[keep,]
detP_filtered <- detP[,keep]
phenoData_filtered <- phenoData[keep,]

###################################################################
#####         DATA NORMALIZATION AND QUALITY CONTROL 2        #####
###################################################################

# Generate a MethylSet object with the methylated and the unmethylated signals (without normalization):
MSetraw <- preprocessRaw(RGSet_filtered)

# Extract and plot quality control information from the MethylSet objects:
qcraw <- getQC(MSetraw)

pdf(file= paste(resultsDirectory, "02-plotQC_revision.pdf", sep = "/"), paper="a4r", height=21, width=28, onefile=TRUE)
par(mfrow=c(1,1))
badSampleCutoff = 9
plotQC(qcraw, badSampleCutoff)
dev.off()

# Remove poor quality samples from R objects:
keep_for_normalization <- rownames(qcraw[rowMeans(as.matrix(qcraw)) > badSampleCutoff,])
RGSet_filtered <- RGSet_filtered[, keep_for_normalization]
targets_filtered <- targets_filtered[match(keep_for_normalization, targets_filtered$SlideArray),]
detP_filtered <- detP[,keep_for_normalization]
phenoData_filtered <- phenoData_filtered[match(keep_for_normalization, targets$SlideArray),]
#write.table(phenoData_filtered, file= paste(tableDirectory, "phenoData_goodsamples.txt", sep = "/"), row.names=TRUE, col.names=TRUE, sep="\t")

# Generate a MethylSet object with the methylated and the unmethylated signals (normalized):  
MSetsq <- preprocessNoob(RGSet_filtered, offset = 15, dyeCorr = TRUE, dyeMethod =  "single")
rm(RGSet, RGSet_filtered, MSetraw, detP)
gc()

# Visualize the data after normalization:
pdf(file= paste(resultsDirectory, "03-norm_step_revision.pdf", sep = "/"), paper="a4r", height=21, width=28, onefile=TRUE)
par(mfrow=c(1,2))
densityPlot(getBeta(MSetsq), sampGroups=phenoData_filtered$Sample_Name, main="Noob normalization", legend=FALSE, pal=phenoData_filtered$color)
legend("topleft", legend=levels(groups), fill=levels(pal), bg="white")

# Visualize the density bean plots (MSetRaw vs MSetsq objects) before and after normalization:
par(mfrow=c(1,2))
densityBeanPlot(getBeta(MSetsq), sampGroups=phenoData_filtered$Sample_Group, sampNames = phenoData_filtered$Sample_Name, main = "Beta Noob norm data", pal = phenoData_filtered$color) 

# Visualize the density M Plots getM(MSetsq)
par(mfrow=c(1,2))
densityPlot(getBeta(MSetsq), sampGroups=phenoData_filtered$Sample_Name, main="Noob norm - Beta values",  legend=FALSE, pal=phenoData_filtered$color)
legend("topleft", legend=levels(groups), fill=levels(pal), bg="white")
densityPlot(getM(MSetsq), sampGroups=phenoData_filtered$Sample_Name, main="Noob norm - M values", legend=FALSE, pal=phenoData_filtered$color)
legend("topleft", legend=levels(groups), fill=levels(pal), bg="white")

dev.off()

###################################################################
#####                    PROBE FILTERING                      #####
###################################################################

# Remove any probes that have failed in one or more samples:
#keep2 <- rowSums(detP_filtered < 0.01) == ncol (MSetsq)

# Remove any probes that have failed in more than 1% samples to avoid excluding CpGs because of our large sample

keep2 <- rowSums(detP_filtered < 0.01) >= 0.99*ncol (MSetsq)
MSetsqFlt <- MSetsq[keep2,]

# If the data includes males and females, remove probes on sex chromosomes:
keep3 <- !(featureNames(MSetsqFlt) %in% annotationEPIC$Name[annotationEPIC$chr %in% c("chrX", "chrY")])
MSetsqFlt <- MSetsqFlt[keep3,]

# Remove probes with SNPs at CpG sites:  Obtained from https://github.com/sirselim/illumina450k_filtering
xProbesinSNPs <- read.csv(file=paste(dataDirectory, " Probes_overlapping_genetic_variants_EPIC.csv", sep="/"), stringsAsFactors=FALSE)  
keep5 <- !(featureNames(MSetsqFlt) %in% xProbesinSNPs$PROBE)   # Watch out!!, download and include annotation for EPIC array
MSetsqFlt <- MSetsqFlt[keep5,]

# Exclude cross reactive probes:  Obtained from https://github.com/sirselim/illumina450k_filtering
xReactiveProbes <- read.csv(file=paste(dataDirectory, " Cross-reactive_probes_on_EPIC_array.csv", sep="/"), stringsAsFactors=FALSE)  
keep6 <- !(featureNames(MSetsqFlt) %in% xReactiveProbes$X)   
MSetsqFlt <- MSetsqFlt[keep6,]

# Exclude multi-mapping probes:  450K, Obtained from https://github.com/sirselim/illumina450k_filtering
multimapProbes <- read.csv(file=paste(dataDirectory, "HumanMethylation450_15017482_v.1.1_hg19_bowtie_multimap.txt", sep="/"), header=FALSE, as.is=TRUE)
keep7 <- !(featureNames(MSetsqFlt) %in% as.character(multimapProbes$V1))   # Watch out!!, download and include annotation for EPIC array
MSetsqFlt <- MSetsqFlt[keep7,]

keeprows <- read.csv(file = "annotation_EPIC.txt", sep = "\t", stringsAsFactors = F)
keep8 <- keeprows$Name
keep8 <- which(names(MSetsqFlt) %in% keep8)
MSetsqFlt <- MSetsqFlt[keep8,]

# Calculate Mvalues and Bvalues for statistical analyses:
mVals <- getM(MSetsqFlt)
bVals <- getBeta(MSetsqFlt)
colnames(mVals) <- phenoData_filtered$Sample_Name
colnames(bVals) <- phenoData_filtered$Sample_Name

# Write and save M and B values:
setwd(tablesDirectory)
write.table(mVals, file = "mVals_CRISPR_exp.txt", sep = "\t", col.names = TRUE, row.names = TRUE, quote = F)
write.table(bVals, file = "bVals_CRISPR_exp.txt", sep = "\t", col.names = TRUE, row.names = TRUE, quote = F)
write.table(phenoData_filtered, file = "phenoData_CRISPR_exp.txt", sep = "\t", col.names = TRUE, row.names = TRUE, quote = F)


# Plot distributions:
bVals_FOSL2 <- melt(bVals[,c(3:4)])
bVals_AF4 <- melt(bVals[,c(1:2)])

ggplot(bVals_AF4, aes(x = variable, y = value, fill = variable)) +
  geom_violin() +
  geom_boxplot(width=0.05, fill = "white") +
  ggtitle("Violin plot of the bValues (MLL-AF4r)") +
  scale_fill_manual(values=c("royalblue1", "plum3")) +
  xlab("Condition") +
  ylab("% Methylation") +
  theme(plot.title = element_text(hjust = 0.5), aspect.ratio = 0.5) +
  theme_bw()

ggplot(bVals_AF4_m, aes(x = variable, y = value, fill = variable)) +
  geom_violin() +
  geom_boxplot(width=0.05, fill = "white") +
  ggtitle("Violin plot of the bValues (MLL-AF4r AP1 motif)") +
  scale_fill_manual(values=c("royalblue1", "plum3")) +
  xlab("Condition") +
  ylab("% Methylation") +
  theme(plot.title = element_text(hjust = 0.5), aspect.ratio = 0.5) +
  theme_bw()


# Get DMPs:
bVals$AF4_vs_FLC <- bVals$AF4 - bVals$FLC
bVals$FOSL2_vs_SEM <- bVals$FOSL2 - bVals$SEM

DMPs_AF4 <- bVals[which(abs(bVals$AF4_vs_FLC) > 0.2),]
DMPs_AF4$Dir <- "hypo"
DMPs_AF4$Dir[which(DMPs_AF4$AF4_vs_FLC > 0)] <- "hyper" 
DMPs_FOSL2 <- bVals[which(abs(bVals$FOSL2_vs_SEM) > 0.4),]
DMPs_FOSL2$Dir <- "hypo"
DMPs_FOSL2$Dir[which(DMPs_FOSL2$FOSL2_vs_SEM > 0)] <- "hyper" 

write.table(DMPs_AF4, file = "List of_CRISPR_t4:11_DMPs.txt", sep = "\t", col.names = TRUE, row.names = TRUE, quote = F)
write.table(DMPs_FOSL2, file = "List of_CRISPR_FOSL2KO_DMPs.txt", sep = "\t", col.names = TRUE, row.names = TRUE, quote = F)

